@@ -38,6 +38,7 @@
 #include "ThreadSearchEvent.h"
 #include "ThreadSearchThread.h"
 #include "ThreadSearchFindData.h"
+#include "ThreadSearchCommon.h"
 #include "ThreadSearchConfPanel.h"
 #include "ThreadSearchControlIds.h"
 #include "wx/tglbtn.h"
@@ -58,7 +59,8 @@ ThreadSearchView::ThreadSearchView(ThreadSearch& threadSearchPlugin)
 {
     m_pFindThread = NULL;
     m_pToolBar    = NULL;
-    const wxString &prefix = GetImagePrefix();
+    const wxString &prefix = GetImagePrefix(false, Manager::Get()->GetAppWindow());
+    const double scaleFactor = cbGetContentScaleFactor(*Manager::Get()->GetAppWindow());
 
     // begin wxGlade: ThreadSearchView::ThreadSearchView
     m_pSplitter = new wxSplitterWindow(this, -1, wxDefaultPosition, wxSize(1,1), wxSP_3D|wxSP_BORDER|wxSP_PERMIT_UNSPLIT);
@@ -72,17 +74,20 @@ ThreadSearchView::ThreadSearchView(ThreadSearch& threadSearchPlugin)
                                       wxDefaultPosition, wxDefaultSize, 0, m_pCboSearchExpr_choices,
                                       wxCB_DROPDOWN|wxTE_PROCESS_ENTER);
     m_pBtnSearch = new wxBitmapButton(this, controlIDs.Get(ControlIDs::idBtnSearch),
-                                      wxBitmap(prefix + wxT("findf.png"), wxBITMAP_TYPE_PNG),
+                                      cbLoadBitmapScaled(prefix + wxT("findf.png"),
+                                                         wxBITMAP_TYPE_PNG, scaleFactor),
                                       wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW);
     m_pBtnOptions = new wxBitmapButton(this, controlIDs.Get(ControlIDs::idBtnOptions),
-                                       wxBitmap(prefix + wxT("options.png"), wxBITMAP_TYPE_PNG),
+                                       cbLoadBitmapScaled(prefix + wxT("options.png"),
+                                                          wxBITMAP_TYPE_PNG, scaleFactor),
                                        wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW);
     m_pStaticLine1 = new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL);
     m_pStaTxtSearchIn = new wxStaticText(this, -1, _("Search in "));
     m_pPnlSearchIn = new SearchInPanel(this, -1);
     m_pStaticLine2 = new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL);
     m_pBtnShowDirItems = new wxBitmapButton(this, controlIDs.Get(ControlIDs::idBtnShowDirItemsClick),
-                                            wxBitmap(prefix + wxT("showdir.png"), wxBITMAP_TYPE_PNG),
+                                            cbLoadBitmapScaled(prefix + wxT("showdir.png"),
+                                                               wxBITMAP_TYPE_PNG, scaleFactor),
                                             wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW);
     m_pPnlDirParams = new DirectoryParamsPanel(&threadSearchPlugin.GetFindData(), this, -1);
     m_pSearchPreview = new cbStyledTextCtrl(m_pPnlPreview, wxID_ANY, wxDefaultPosition, wxSize(1,1));
@@ -292,11 +297,24 @@ void ThreadSearchView::OnQuickOptions(wxCommandEvent &event)
 
 void ThreadSearchView::UpdateOptionsButtonImage(const ThreadSearchFindData &findData)
 {
-    wxString name = (findData.IsOptionEnabled() ? wxT("optionsactive.png") : wxT("options.png"));
-    wxBitmap bitmap(GetImagePrefix() + name, wxBITMAP_TYPE_PNG);
-    m_pBtnOptions->SetBitmapLabel(bitmap);
+    {
+        const wxString name = GetImagePrefix(false, m_pBtnOptions)
+                            + (findData.IsOptionEnabled() ? wxT("optionsactive.png") : wxT("options.png"));
+
+        const double scaleFactor = cbGetContentScaleFactor(*m_pBtnOptions);
+        wxBitmap bitmap=cbLoadBitmapScaled(name, wxBITMAP_TYPE_PNG, scaleFactor);
+        m_pBtnOptions->SetBitmapLabel(bitmap);
+    }
+
     if (m_pToolBar)
+    {
+        const wxString name = GetImagePrefix(true)
+                            + (findData.IsOptionEnabled() ? wxT("optionsactive.png") : wxT("options.png"));
+
+        const double scaleFactor = cbGetContentScaleFactor(*m_pToolBar);
+        wxBitmap bitmap=cbLoadBitmapScaled(name, wxBITMAP_TYPE_PNG, scaleFactor);
         m_pToolBar->SetToolNormalBitmap(controlIDs.Get(ControlIDs::idBtnOptions), bitmap);
+    }
 }
 
 void ThreadSearchView::OnQuickOptionsUpdateUI(wxUpdateUIEvent &event)
@@ -349,19 +367,23 @@ void ThreadSearchView::OnSplitterDoubleClick(wxSplitterEvent &/*event*/)
 
 void ThreadSearchView::set_properties()
 {
-    const wxString &prefix = GetImagePrefix();
+    const wxString &prefix = GetImagePrefix(false, this);
+    const double scaleFactor = cbGetContentScaleFactor(*this);
 
     // begin wxGlade: ThreadSearchView::set_properties
     SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
     m_pCboSearchExpr->SetMinSize(wxSize(180, -1));
     m_pBtnSearch->SetToolTip(_("Search in files"));
-    m_pBtnSearch->SetBitmapDisabled(wxBitmap(prefix + wxT("findfdisabled.png"), wxBITMAP_TYPE_PNG));
+    m_pBtnSearch->SetBitmapDisabled(cbLoadBitmapScaled(prefix + wxT("findfdisabled.png"),
+                                                       wxBITMAP_TYPE_PNG, scaleFactor));
     m_pBtnSearch->SetSize(m_pBtnSearch->GetBestSize());
     m_pBtnOptions->SetToolTip(_("Options"));
-    m_pBtnOptions->SetBitmapDisabled(wxBitmap(prefix + wxT("optionsdisabled.png"), wxBITMAP_TYPE_PNG));
+    m_pBtnOptions->SetBitmapDisabled(cbLoadBitmapScaled(prefix + wxT("optionsdisabled.png"),
+                                                        wxBITMAP_TYPE_PNG, scaleFactor));
     m_pBtnOptions->SetSize(m_pBtnOptions->GetBestSize());
     m_pBtnShowDirItems->SetToolTip(_("Show dir Items"));
-    m_pBtnShowDirItems->SetBitmapDisabled(wxBitmap(prefix + wxT("showdirdisabled.png"), wxBITMAP_TYPE_PNG));
+    m_pBtnShowDirItems->SetBitmapDisabled(cbLoadBitmapScaled(prefix + wxT("showdirdisabled.png"),
+                                                             wxBITMAP_TYPE_PNG, scaleFactor));
     m_pBtnShowDirItems->SetSize(m_pBtnShowDirItems->GetBestSize());
     m_pPnlPreview->SetMinSize(wxSize(25, -1));
     // end wxGlade
@@ -533,7 +555,8 @@ bool ThreadSearchView::UpdatePreview(const wxString& file, long line)
         // Colorize
         cbEditor::ApplyStyles(m_pSearchPreview);
         EditorColourSet EdColSet;
-        EdColSet.Apply(EdColSet.GetLanguageForFilename(m_PreviewFilePath), m_pSearchPreview);
+        EdColSet.Apply(EdColSet.GetLanguageForFilename(m_PreviewFilePath), m_pSearchPreview, false,
+                       true);
 
         SetFoldingIndicator(mgr->ReadInt(_T("/folding/indicator"), 2));
         UnderlineFoldedLines(mgr->ReadBool(_T("/folding/underline_folded_line"), true));
@@ -914,33 +937,45 @@ bool ThreadSearchView::IsSearchRunning()
 void ThreadSearchView::UpdateSearchButtons(bool enable, eSearchButtonLabel label)
 {
     // Labels and pictures paths
-    wxString searchButtonLabels[]        = {_("Search"), _("Cancel search"), wxEmptyString};
+    wxString searchButtonLabels[] = {_("Search"), _("Cancel search"), wxEmptyString};
 
-    ConfigManager *cfg = Manager::Get()->GetConfigManager(_T("app"));
-    int toolbar_size = cfg->ReadBool(_T("/environment/toolbar_size"),true)?1:0;
-    wxString prefix = ConfigManager::GetDataFolder() + _T("/images/ThreadSearch/") + (toolbar_size==1?_T("16x16/"):_T("22x22/"));
-
-    wxString searchButtonPathsEnabled[]  = {prefix + wxT("findf.png"),
-                                            prefix + wxT("stop.png") ,
+    wxString searchButtonPathsEnabled[]  = {wxT("findf.png"),
+                                            wxT("stop.png") ,
                                             wxEmptyString};
 
-    wxString searchButtonPathsDisabled[] = {prefix + wxT("findfdisabled.png"),
-                                            prefix + wxT("stopdisabled.png") ,
+    wxString searchButtonPathsDisabled[] = {wxT("findfdisabled.png"),
+                                            wxT("stopdisabled.png") ,
                                             wxEmptyString};
 
     // Gets toolbar search button pointer
     // Changes label/bitmap only if requested
-    if ( label != skip )
+    if (label != skip)
     {
-        m_pBtnSearch->SetToolTip(searchButtonLabels[label]);
-        m_pBtnSearch->SetBitmapLabel   (wxBitmap(searchButtonPathsEnabled [label], wxBITMAP_TYPE_PNG));
-        m_pBtnSearch->SetBitmapDisabled(wxBitmap(searchButtonPathsDisabled[label], wxBITMAP_TYPE_PNG));
-        //{ Toolbar buttons
-        m_pToolBar->SetToolNormalBitmap(controlIDs.Get(ControlIDs::idBtnSearch),
-                                        wxBitmap(searchButtonPathsEnabled [label], wxBITMAP_TYPE_PNG));
-        m_pToolBar->SetToolDisabledBitmap(controlIDs.Get(ControlIDs::idBtnSearch),
-                                          wxBitmap(searchButtonPathsDisabled[label], wxBITMAP_TYPE_PNG));
-        //}
+        {
+            const wxString &prefix = GetImagePrefix(false, m_pBtnSearch);
+            const double scaleFactor = cbGetContentScaleFactor(*m_pBtnSearch);
+            wxBitmap bmpSearch=cbLoadBitmapScaled(prefix + searchButtonPathsEnabled[label],
+                                                  wxBITMAP_TYPE_PNG, scaleFactor);
+            wxBitmap bmpSearchDisabled=cbLoadBitmapScaled(prefix + searchButtonPathsDisabled[label],
+                                                          wxBITMAP_TYPE_PNG, scaleFactor);
+
+            m_pBtnSearch->SetToolTip(searchButtonLabels[label]);
+            m_pBtnSearch->SetBitmapLabel(bmpSearch);
+            m_pBtnSearch->SetBitmapDisabled(bmpSearchDisabled);
+        }
+
+        {
+            //Toolbar buttons
+            const wxString &prefix = GetImagePrefix(true);
+            const double scaleFactor = cbGetContentScaleFactor(*m_pToolBar);
+            wxBitmap bmpSearch=cbLoadBitmapScaled(prefix + searchButtonPathsEnabled[label],
+                                                  wxBITMAP_TYPE_PNG, scaleFactor);
+            wxBitmap bmpSearchDisabled=cbLoadBitmapScaled(prefix + searchButtonPathsDisabled[label],
+                                                          wxBITMAP_TYPE_PNG, scaleFactor);
+            m_pToolBar->SetToolNormalBitmap(controlIDs.Get(ControlIDs::idBtnSearch), bmpSearch);
+            m_pToolBar->SetToolDisabledBitmap(controlIDs.Get(ControlIDs::idBtnSearch),
+                                              bmpSearchDisabled);
+        }
     }
 
     // Sets enable state
@@ -948,13 +983,22 @@ void ThreadSearchView::UpdateSearchButtons(bool enable, eSearchButtonLabel label
     m_pToolBar->EnableTool(controlIDs.Get(ControlIDs::idBtnSearch), enable);
 }
 
-wxString ThreadSearchView::GetImagePrefix() const
+wxString GetImagePrefix(bool toolbar, wxWindow *window)
 {
-    ConfigManager *cfg = Manager::Get()->GetConfigManager(wxT("app"));
-    if (cfg->ReadBool(wxT("/environment/toolbar_size"), true))
-        return ConfigManager::GetDataFolder() + wxT("/images/ThreadSearch/16x16/");
+    if (toolbar)
+    {
+        const int size = Manager::Get()->GetImageSize(Manager::UIComponent::Toolbars);
+        return ConfigManager::GetDataFolder()
+            + wxString::Format(wxT("/ThreadSearch.zip#zip:images/%dx%d/"), size, size);
+    }
     else
-        return ConfigManager::GetDataFolder() + wxT("/images/ThreadSearch/22x22/");
+    {
+        cbAssert(window != nullptr);
+        const int targetHeight = floor(16 * cbGetActualContentScaleFactor(*window));
+        const int size = cbFindMinSize16to64(targetHeight);
+        return ConfigManager::GetDataFolder()
+            + wxString::Format(wxT("/ThreadSearch.zip#zip:images/%dx%d/"), size, size);
+    }
 }
 
 void ThreadSearchView::ShowSearchControls(bool show)
