@@ -17,7 +17,7 @@
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
-// RCS-ID: $Id: codesnippetstreectrl.cpp 10874 2016-07-16 20:00:28Z jenslody $
+// RCS-ID: $Id: codesnippetstreectrl.cpp 11536 2018-12-20 11:16:28Z fuscated $
 
 #ifdef WX_PRECOMP
     #include "wx_pch.h"
@@ -49,7 +49,6 @@
 #include "codesnippetstreectrl.h"
 #include "codesnippetswindow.h"
 #include "snippetsconfig.h"
-#include "GenericMessageBox.h"
 #include "menuidentifiers.h"
 #include "snippetsconfig.h"
 #include "FileImport.h"
@@ -113,7 +112,7 @@ CodeSnippetsTreeCtrl::CodeSnippetsTreeCtrl(wxWindow *parent, const wxWindowID id
 CodeSnippetsTreeCtrl::~CodeSnippetsTreeCtrl()
 // ----------------------------------------------------------------------------
 {
-    //dtor
+    delete m_pDragCursor;
     // Tell all that TreeCtrl is gone;
     GetConfig()->SetSnippetsTreeCtrl(0);
 
@@ -136,8 +135,7 @@ bool CodeSnippetsTreeCtrl::IsFileSnippet (wxTreeItemId treeItemId  )
         //-LOGIT( _T("$macros name[%s]"),fileName.c_str() );
     //-#endif
 
-    if ( not ::wxFileExists( fileName) ) return false;
-    return true;
+    return ::wxFileExists(fileName);
 }
 // ----------------------------------------------------------------------------
 bool CodeSnippetsTreeCtrl::IsFileLinkSnippet (wxTreeItemId treeItemId  )
@@ -160,9 +158,7 @@ bool CodeSnippetsTreeCtrl::IsFileLinkSnippet (wxTreeItemId treeItemId  )
     {   // if text is > 128 characters, not a filelink.
         return false;
     }
-    if ( not ::wxFileExists( fileName) ) return false;
-    return true;
-
+    return ::wxFileExists(fileName);
 }//IsFileLinkSnippet
 // ----------------------------------------------------------------------------
 wxString CodeSnippetsTreeCtrl::GetFileLinkExt (wxTreeItemId treeItemId  )
@@ -170,11 +166,11 @@ wxString CodeSnippetsTreeCtrl::GetFileLinkExt (wxTreeItemId treeItemId  )
 {
     //return filename extension
 
-    if ( not IsFileLinkSnippet(treeItemId) ) return wxT("");
+    if ( not IsFileLinkSnippet(treeItemId) ) return wxEmptyString;
     wxTreeItemId itemId = treeItemId;
     if ( itemId == (void*)0) itemId = GetSelection();
-    if (not itemId.IsOk()) return wxT("");
-    if (not IsSnippet(itemId) ) return wxT("");
+    if (not itemId.IsOk()) return wxEmptyString;
+    if (not IsSnippet(itemId) ) return wxEmptyString;
     wxString fileName = GetSnippetString(itemId).BeforeFirst('\r');
     fileName = fileName.BeforeFirst('\n');
     // substitute $macros with actual text
@@ -184,7 +180,7 @@ wxString CodeSnippetsTreeCtrl::GetFileLinkExt (wxTreeItemId treeItemId  )
             Manager::Get()->GetMacrosManager()->ReplaceMacros(fileName);
         //-LOGIT( _T("$macros name[%s]"),fileName.c_str() );
     //-#endif
-    if ( not ::wxFileExists( fileName) ) return wxT("");
+    if ( not ::wxFileExists( fileName) ) return wxEmptyString;
     wxFileName filename(fileName);
     return filename.GetExt();
 }
@@ -194,6 +190,7 @@ void CodeSnippetsTreeCtrl::OnItemSelectChanging(wxTreeEvent& event)
 {
     // UNUSED
     //-if (m_pTopDialog) event.Veto();
+    wxUnusedVar(event);
     return;
 }
 // ----------------------------------------------------------------------------
@@ -212,6 +209,7 @@ void CodeSnippetsTreeCtrl::OnItemSelected(wxTreeEvent& event)
 	if (const SnippetTreeItemData* eventItem =
         (SnippetTreeItemData*)GetItemData(event.GetItem()))
 	{
+/*
         wxString itemDescription;
 		// Check the type of the item and add the menu items
 		switch (eventItem->GetType())
@@ -226,7 +224,7 @@ void CodeSnippetsTreeCtrl::OnItemSelected(wxTreeEvent& event)
 			//-SetStatusText( GetSnippetDescription(itemId) );
 			break;
 		}
-
+*/
 		// Save the item ID for later use
 		m_MnuAssociatedItemID = eventItem->GetId();
 
@@ -611,12 +609,12 @@ void CodeSnippetsTreeCtrl::LoadItemsFromXmlNode(const TiXmlElement* node, const
 			}
 			else
 			{
-                GenericMessageBox(_T("CodeSnippets: Error loading XML file; element \"snippet\" cannot be found."));
+                wxMessageBox(_T("CodeSnippets: Error loading XML file; element \"snippet\" cannot be found."));
 			}
 		}
 		else
 		{
-		    GenericMessageBox(_T("CodeSnippets: Error loading XML file; attribute \"type\" is \"") + itemType + _T("\" which is invalid item type."));
+		    wxMessageBox(_T("CodeSnippets: Error loading XML file; attribute \"type\" is \"") + itemType + _T("\" which is invalid item type."));
 			return;
 		}
 	} // end for
@@ -712,9 +710,9 @@ bool CodeSnippetsTreeCtrl::LoadItemsFromFile(const wxString& fileName, bool bApp
 		   else //IsApplication
 		   {
                 //-wxMessageBox(_T("CodeSnippets: Cannot load file \"") + fileName + _T("\": ") + csC2U(doc.ErrorDesc()));
-                GenericMessageBox(_T("CodeSnippets: Cannot load file \"") + fileName + _T("\": ") + csC2U(doc.ErrorDesc()));
+                wxMessageBox(_T("CodeSnippets: Cannot load file \"") + fileName + _T("\": ") + csC2U(doc.ErrorDesc()));
                 //-wxMessageBox(_T("CodeSnippets: Backup of the failed file has been created."));
-                GenericMessageBox(_T("CodeSnippets: Backup of the failed file has been created."));
+                wxMessageBox(_T("CodeSnippets: Backup of the failed file has been created."));
 		   }
 		   //-#endif
 		}
@@ -837,14 +835,14 @@ bool CodeSnippetsTreeCtrl::RemoveItem(const wxTreeItemId RemoveItemId)
     if (trashItem || shiftKeyIsDown )
     {
         // if FileLink, memorize the filename
-        wxString filename = wxEmptyString;
+        wxString filename;
         if ( IsFileSnippet(itemId) )
            filename = GetSnippetFileLink(itemId);
 
         // if this was a FileLink, ask if user wants to delete file
         if ( not filename.IsEmpty() ) {
-            int answer = GenericMessageBox( wxT("Delete physical file?\n\n")+filename,
-                                                    wxT("Delete"),wxYES_NO );
+            int answer = wxMessageBox( wxT("Delete physical file?\n\n")+filename,
+                                      wxT("Delete"),wxYES_NO );
             if ( answer == wxYES)
                 /*int done =*/ ::wxRemoveFile(filename);
         }
@@ -869,7 +867,6 @@ void CodeSnippetsTreeCtrl::SetSnippetImage(wxTreeItemId itemId)
         SetItemImage( itemId, TREE_IMAGE_SNIPPET_URL);
     else
         SetItemImage( itemId, TREE_IMAGE_SNIPPET_TEXT);
-    return;
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::FetchFileModificationTime(wxDateTime savedTime)
@@ -906,8 +903,6 @@ void CodeSnippetsTreeCtrl::OnBeginTreeItemDrag(wxTreeEvent& event)
         m_bBeginInternalDrag = 0;
 
     event.Allow();
-
-    return;
 }//OnBeginTreeItemDrag
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::EndInternalTreeItemDrag()
@@ -975,9 +970,6 @@ void CodeSnippetsTreeCtrl::EndInternalTreeItemDrag()
     m_itemAtKeyDown = 0;
     m_itemAtKeyUp = 0;
     #endif
-
-    return;
-
 }//OnEndTreeItemDrag
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::OnEnterWindow(wxMouseEvent& event)
@@ -1026,7 +1018,7 @@ void CodeSnippetsTreeCtrl::OnLeaveWindow(wxMouseEvent& event)
     // User has dragged mouse out of source window.
     // if us is dragging mouse, save the source item for later use
     // in FinishDrag()
-
+    wxUnusedVar(event);
     #ifdef LOGGING
      //LOGIT( _T("CodeSnippetsTreeCtrl::OnLeaveWindow") );
     #endif //LOGGING
@@ -1059,8 +1051,6 @@ void CodeSnippetsTreeCtrl::OnLeaveWindow(wxMouseEvent& event)
         SendMouseLeftUp(this, m_TreeMousePosn.x, m_TreeMousePosn.y);
     }
     #endif //__WXMSW__
-
-    return;
 }//OnLeaveWindow
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::FinishExternalDrag()
@@ -1116,11 +1106,9 @@ void CodeSnippetsTreeCtrl::FinishExternalDrag()
                 textData->GetText().c_str(),
                 fileData->GetFilenames().Item(0).c_str() );
     #endif //LOGGING
-        // allow both copy and move
-    int flags = 0;
-    flags |= wxDrag_AllowMove;
+
     // do the dragNdrop
-    wxDragResult result = source.DoDragDrop(flags);
+    wxDragResult result = source.DoDragDrop(wxDrag_AllowMove); // allow both copy and move
 
     // report the results
     #if LOGGING
@@ -1211,7 +1199,6 @@ void CodeSnippetsTreeCtrl::OnMouseWheelEvent(wxMouseEvent& event)
         ctrlFont.SetPointSize( ctrlFont.GetPointSize()+1);
 
     SetFont(ctrlFont);
-    return;
 }
 // ----------------------------------------------------------------------------
 wxTreeItemId CodeSnippetsTreeCtrl::ConvertSnippetToCategory(wxTreeItemId itemId)
@@ -1419,8 +1406,6 @@ void CodeSnippetsTreeCtrl::EditSnippetAsFileLink()
         #endif //LOGGING
         ::wxExecute( execString);
     }
-
-    return;
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::OpenSnippetAsFileLink()
@@ -1443,8 +1428,6 @@ void CodeSnippetsTreeCtrl::OpenSnippetAsFileLink()
 
     // user requested "MIME" type to open file
     EditSnippetWithMIME();
-
-    return;
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::EditSnippetAsText()
@@ -1476,7 +1459,7 @@ void CodeSnippetsTreeCtrl::EditSnippetAsText()
         if (GetConfig()->IsApplication() ) msg = msg + wxT("Use Menu->");
         else msg = msg + wxT("Right click Root item. Use ");
         msg = msg + wxT("Settings to set a better editor.\n");
-        GenericMessageBox( msg );
+        wxMessageBox( msg );
     }
 
     // let user edit the snippet text
@@ -1499,8 +1482,6 @@ void CodeSnippetsTreeCtrl::EditSnippetAsText()
     else
         EditSnippet(pSnippetTreeItemData);
 
-
-    return;
 }//EditSnippetAsText
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::SaveSnippetAsFileLink()
@@ -1523,7 +1504,7 @@ void CodeSnippetsTreeCtrl::SaveSnippetAsFileLink()
     // if file already exists preserve the old data
     if ( ::wxFileExists( fileName ) )
     {   // item snippet is already a filename
-        answer = GenericMessageBox(
+        answer = wxMessageBox(
             wxT("Item is already a file link named:\n")+fileName
                 + wxT(" \n\nAre you sure you want to rewrite the file?\n"),
             wxT("Warning"),wxYES|wxNO); //, GetMainFrame(), mousePosn.x, mousePosn.y);
@@ -1531,7 +1512,8 @@ void CodeSnippetsTreeCtrl::SaveSnippetAsFileLink()
         {   // read data from old file
             wxFile oldFile( fileName, wxFile::read);
             if (not oldFile.IsOpened() )
-            {   GenericMessageBox(wxT("Abort.Error reading data file."));
+            {
+                wxMessageBox(wxT("Abort. Error reading data file."));
                 return;
             }
             unsigned long fileSize = oldFile.Length();
@@ -1587,7 +1569,7 @@ void CodeSnippetsTreeCtrl::SaveSnippetAsFileLink()
     wxFile newFile( newFileName, wxFile::write);
     if (not newFile.IsOpened() )
     {
-        GenericMessageBox(wxT("Open failed for:")+newFileName);
+        wxMessageBox(wxT("Open failed for:")+newFileName);
         return ;
     }
     newFile.Write( csU2C(snippetData), snippetData.Length());
@@ -1596,12 +1578,8 @@ void CodeSnippetsTreeCtrl::SaveSnippetAsFileLink()
     SetSnippetString( newFileName );
 
     // verify the item tree image
-    if ( IsFileSnippet() )
-        SetItemImage( itemId, TREE_IMAGE_SNIPPET_FILE);
-    else
-        SetItemImage( itemId, TREE_IMAGE_SNIPPET_TEXT);
+    SetItemImage( itemId, IsFileSnippet() ? TREE_IMAGE_SNIPPET_FILE : TREE_IMAGE_SNIPPET_TEXT);
 
-    return;
 }//OnMnuSaveAsFile
 // ----------------------------------------------------------------------------
 bool CodeSnippetsTreeCtrl::EditSnippetProperties(wxTreeItemId& itemId)
@@ -1615,17 +1593,15 @@ bool CodeSnippetsTreeCtrl::EditSnippetProperties(wxTreeItemId& itemId)
 
     result = ExecuteDialog(pdlg, waitSem);
     // Save any changed data
-	if ( result == wxID_OK )
-	{
+    if ( result == wxID_OK )
+    {
         // verify the item tree image
         SetSnippetImage(itemId);
-	}
-    if (result == wxID_OK) {
         SetFileChanged(true);
     }
 
     pdlg->Destroy();
-    return (result = (result==wxID_OK));
+    return (result == wxID_OK);
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::EditSnippet(SnippetTreeItemData* pSnippetTreeItemData, wxString fileName)
@@ -1633,6 +1609,9 @@ void CodeSnippetsTreeCtrl::EditSnippet(SnippetTreeItemData* pSnippetTreeItemData
 {
 
     // Create editor for snippet item
+
+    wxUnusedVar(fileName);
+    wxUnusedVar(pSnippetTreeItemData);
     if ((SnippetTreeItemData*)( GetItemData(GetAssociatedItemID() )))
     {
         // Get the snippet text associated with this tree id
@@ -1801,11 +1780,10 @@ void CodeSnippetsTreeCtrl::EditSnippetWithMIME()
     // Do not use these when using global wxTheMimeTypesManager pointer
     ////if ( m_mimeDatabase ) delete m_mimeDatabase;
     ////m_mimeDatabase = 0;
-
-    return;
 }
 // ----------------------------------------------------------------------------
-int CodeSnippetsTreeCtrl::ExecuteDialog(wxScrollingDialog* pdlg, wxSemaphore& waitSem)
+//int CodeSnippetsTreeCtrl::ExecuteDialog(wxScrollingDialog* pdlg, wxSemaphore& waitSem)
+int CodeSnippetsTreeCtrl::ExecuteDialog(SnippetProperty* pdlg, wxSemaphore& waitSem)
 // ----------------------------------------------------------------------------
 {
     if (m_pPropertiesDialog) return 0;
@@ -1845,6 +1823,7 @@ int CodeSnippetsTreeCtrl::ExecuteDialog(wxScrollingDialog* pdlg, wxSemaphore& wa
                 wxYield();
             }
             retcode = pdlg->GetReturnCode();
+
         }
         // Release main apps closeWindow function
         GetConfig()->GetMainFrame()->Disconnect( wxEVT_CLOSE_WINDOW,
@@ -1976,14 +1955,13 @@ void CodeSnippetsTreeCtrl::OnIdle()
 
     ////event.Skip(); this routine is called from another OnIdle which does the
     // event.Skip() itself.
-    return;
 }//OnIdle
 
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::OnShutdown(wxCloseEvent& event)
 // ----------------------------------------------------------------------------
 {
-    event.Skip(); return;
+    event.Skip();
 }
 // ----------------------------------------------------------------------------
 wxTreeItemId CodeSnippetsTreeCtrl::ResetSnippetsIDs(const wxTreeItemId& startNode)
@@ -2095,7 +2073,6 @@ void CodeSnippetsTreeCtrl::CreateDirLevels(const wxString& pathNameIn)
 {
     // FileImport Traverser will create any missing directories
     FileImportTraverser fit(_T("dummy"), pathNameIn);
-    return;
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsTreeCtrl::SendMouseLeftUp(const wxWindow* pWin, const int mouseX, const int mouseY)
@@ -2122,7 +2099,7 @@ void CodeSnippetsTreeCtrl::SendMouseLeftUp(const wxWindow* pWin, const int mouse
         // move mouse into the window
         MSW_MouseMove( fullScreen.x, fullScreen.y );
         // send mouse LeftKeyUp
-        INPUT Input         = {0};
+        INPUT Input         = {0,0,0,0,0,0,0};
         Input.type          = INPUT_MOUSE;
         Input.mi.dwFlags    = MOUSEEVENTF_LEFTUP;
         ::SendInput(1,&Input,sizeof(INPUT));
@@ -2141,7 +2118,7 @@ void CodeSnippetsTreeCtrl::MSW_MouseMove(int x, int y )
       double fScreenHeight  = ::GetSystemMetrics( SM_CYSCREEN )-1;
       double fx = x*(65535.0f/fScreenWidth);
       double fy = y*(65535.0f/fScreenHeight);
-      INPUT  Input={0};
+      INPUT  Input={0,0,0,0,0,0,0};
       Input.type      = INPUT_MOUSE;
       Input.mi.dwFlags  = MOUSEEVENTF_MOVE|MOUSEEVENTF_ABSOLUTE;
       Input.mi.dx = (LONG)fx;
