@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
-// RCS-ID: $Id: codesnippetswindow.cpp 11066 2017-05-27 20:29:39Z d_anselmi $
+// RCS-ID: $Id: codesnippetswindow.cpp 11352 2018-03-27 22:00:55Z fuscated $
 
 #ifdef WX_PRECOMP //
     #include "wx_pch.h"
@@ -66,7 +66,6 @@
 #include "snippetsconfig.h"
 #include "snippetsimages.h"
 #include "codesnippetstreectrl.h"
-#include "GenericMessageBox.h"
 #include "settingsdlg.h"
 #include "menuidentifiers.h"
 
@@ -508,8 +507,7 @@ void CodeSnippetsWindow::OnItemActivated(wxTreeEvent& event)
         return;
     }
 
-    OnMnuEditSnippet( ev )        ;
-    return;
+    OnMnuEditSnippet( ev );
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsWindow::OnItemMenu(wxTreeEvent& event)
@@ -699,6 +697,8 @@ void CodeSnippetsWindow::OnMnuRename(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
     // Get the associated item id
+
+    wxUnusedVar(event);
     wxTreeItemId itemID = GetAssociatedItemID();
 
     CodeSnippetsTreeCtrl* pTree = GetSnippetsTreeCtrl();
@@ -908,6 +908,8 @@ void CodeSnippetsWindow::OnMnuLoadSnippetsFromFile(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
     // Allow user to specify new filename or old file to load
+
+    wxUnusedVar(event);
     //-wxFileDialog dlg(this, _("Load snippets from file"), wxEmptyString, wxEmptyString, _("XML files (*.xml)|*.xml|All files (*.*)|*.*"), wxOPEN|wxFILE_MUST_EXIST);
     wxFileDialog dlg(this, _("Load snippets from file"), wxEmptyString, wxEmptyString,
         _("XML files (*.xml)|*.xml|All files (*.*)|*.*"), wxFD_OPEN);
@@ -950,21 +952,18 @@ void CodeSnippetsWindow::OnMnuFileBackup(wxCommandEvent& event)
     if (GetFileChanged() )
         OnMnuSaveSnippets(event);
     const wxString IndexFile = GetConfig()->SettingsSnippetsXmlPath;
-    wxString bkupName = wxEmptyString;
-    unsigned i = 0;
-    while (true)
+    wxString bkupName;
+    for (unsigned i = 1; ; ++i)
     {
-        ++i;
         bkupName = IndexFile;
         bkupName << wxT(".")  << i;
-        if ( ::wxFileExists(bkupName) ) {continue;}
-        break;
-    }//while
+        if (!::wxFileExists(bkupName))
+            break;
+    }
     int done = ::wxCopyFile(IndexFile, bkupName);
-    GenericMessageBox( wxString::Format( wxT("Backup %s for\n\n %s"),
-                done?wxT("succeeded"):wxT("failed"),
-                bkupName.c_str()) );
-
+    wxMessageBox( wxString::Format( wxT("Backup %s for\n\n %s"),
+                                   done?wxT("succeeded"):wxT("failed"),
+                                   bkupName.c_str()) );
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsWindow::OnEndLabelEdit(wxTreeEvent& event)
@@ -987,6 +986,7 @@ void CodeSnippetsWindow::OnMnuRemoveAll(wxCommandEvent& /*event*/)
 void CodeSnippetsWindow::OnMnuCaseSensitive(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
+    wxUnusedVar(event);
     GetConfig()->m_SearchConfig.caseSensitive = (not GetConfig()->m_SearchConfig.caseSensitive);
 }
 // ----------------------------------------------------------------------------
@@ -1011,6 +1011,8 @@ void CodeSnippetsWindow::OnMnuClear(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
     // search->clear context menu item
+
+    wxUnusedVar(event);
     m_SearchSnippetCtrl->Clear();
 
 }
@@ -1018,6 +1020,7 @@ void CodeSnippetsWindow::OnMnuClear(wxCommandEvent& event)
 void CodeSnippetsWindow::OnMnuSearchExtended(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
+    wxUnusedVar(event);
     wxWindow* m_pAppWindow = Manager::Get()->GetAppWindow();
     if ( not m_pAppWindow ) m_pAppWindow = wxTheApp->GetTopWindow();
 
@@ -1033,6 +1036,7 @@ void CodeSnippetsWindow::OnMnuSearchExtended(wxCommandEvent& event)
 void CodeSnippetsWindow::OnMnuCopyToClipboard(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
+    wxUnusedVar(event);
     if (wxTheClipboard->Open())
     {
         const SnippetTreeItemData* itemData = (SnippetTreeItemData*)(GetSnippetsTreeCtrl()->GetItemData(GetAssociatedItemID()));
@@ -1330,17 +1334,10 @@ void CodeSnippetsWindow::CheckForExternallyModifiedFiles()
     if (b_modified)
     {
         // modified; ask to reload
-        int ret = -1;
-        {
-            wxString msg;
-            msg.Printf(_("%s\n\nFile is modified outside the IDE...\nDo you want to reload it (you will lose any unsaved work)?"),
-                       GetConfig()->SettingsSnippetsXmlPath.c_str());
-            if (GenericMessageBox(msg, whichApp + _("needs to Reload file?!"), wxICON_QUESTION | wxYES_NO) == wxYES)
-                ret = wxYES;
-            else
-                ret = wxNO;
-
-        }
+        wxString msg;
+        msg.Printf(_("%s\n\nFile is modified outside the IDE...\nDo you want to reload it (you will lose any unsaved work)?"),
+                   GetConfig()->SettingsSnippetsXmlPath.c_str());
+        int ret = wxMessageBox(msg, whichApp + _("needs to Reload file?!"), wxICON_QUESTION | wxYES_NO);
         if ( ret == wxYES )
         {
             if (not GetSnippetsTreeCtrl()->LoadItemsFromFile(GetConfig()->SettingsSnippetsXmlPath, m_AppendItemsFromFile))
@@ -1348,7 +1345,7 @@ void CodeSnippetsWindow::CheckForExternallyModifiedFiles()
                 // File modification time is saved by LoadItemsFromFile
                 //GetSnippetsTreeCtrl()->FetchFileModificationTime(last);
         }
-        else if (ret == wxNO)
+        else
             GetSnippetsTreeCtrl()->FetchFileModificationTime();
     }
 
@@ -1356,7 +1353,7 @@ void CodeSnippetsWindow::CheckForExternallyModifiedFiles()
     {
         wxString msg;
         msg.Printf(_("Could not reload file:\n\n%s"), GetConfig()->SettingsSnippetsXmlPath.c_str() );
-        GenericMessageBox(msg, whichApp +  _("Error"), wxICON_ERROR);
+        wxMessageBox(msg, whichApp +  _("Error"), wxICON_ERROR);
     }
     m_isCheckingForExternallyModifiedFiles = false;
 
@@ -1383,12 +1380,12 @@ void CodeSnippetsWindow::OnIdle(wxIdleEvent& event)
 // ----------------------------------------------------------------------------
 {
     event.Skip();
-    return;
 }
 // ----------------------------------------------------------------------------
 void CodeSnippetsWindow::OnMnuAbout(wxCommandEvent& event)
 // ----------------------------------------------------------------------------
 {
+    wxUnusedVar(event);
     wxString wxbuild(wxVERSION_STRING);
 
     #if defined(__WXMSW__)
@@ -1403,10 +1400,8 @@ void CodeSnippetsWindow::OnMnuAbout(wxCommandEvent& event)
         wxbuild << _T("-ANSI build");
     #endif // wxUSE_UNICODE
 
-    wxString buildInfo = wxbuild;
-    wxString
-        pgmVersionString = wxT("CodeSnippets v") + GetConfig()->GetVersion();
-    buildInfo = wxT("\t")+pgmVersionString + wxT("\n")+ wxT("\t")+buildInfo;
+    wxString pgmVersionString = wxT("CodeSnippets v") + GetConfig()->GetVersion();
+    wxString buildInfo = wxT("\t")+pgmVersionString + wxT("\n") + wxT("\t") + wxbuild;
     buildInfo = buildInfo + wxT("\n\n\t")+wxT("Original Code by Arto Jonsson");
     buildInfo = buildInfo + wxT("\n\t")+wxT("Modified/Enhanced by Pecan Heber");
 
@@ -1418,6 +1413,7 @@ void CodeSnippetsWindow::OnMnuTest(wxCommandEvent& event)
 {
     // Used to issue assorted tests during debugging
 
+    wxUnusedVar(event);
     #if defined(LOGGING)
     LOGIT( _T("Test::EVT_S_DRAGSCROLL_EVENT[%d]"),event.GetId() );
     #endif
@@ -1441,35 +1437,33 @@ void CodeSnippetsWindow::ShowSnippetsAbout(wxString buildInfo)
 
     //wxString msg = wxbuildinfo(long_f);
     wxString helpText;
-    helpText << wxT("\n\n Each Snippet item may specify either text or a File Link.\n")
-             << wxT("\n")
-             << wxT(" Snippets may be edited via the context menu \n")
-             << wxT("\n")
+    helpText << wxT("\n\n")
+             << wxT("Each Snippet item may specify either text or a File Link.")
+             << wxT("\n\n")
+             << wxT("Snippets may be edited via the context menu")
+             << wxT("\n\n")
 
-             << wxT(" File Link snippets are created by dragging text to a new snippet, \n")
-             << wxT(" then using the context menu to \"Convert to File Link\". \n")
-             << wxT(" The data will be written to the specified file and the filename \n")
-             << wxT(" will be placed in the snippets text area as a Link. \n")
-             << wxT("\n")
+             << wxT("File Link snippets are created by dragging text to a new snippet, ")
+             << wxT("then using the context menu to \"Convert to File Link\". ")
+             << wxT("The data will be written to the specified file and the filename ")
+             << wxT("will be placed in the snippets text area as a Link.")
+             << wxT("\n\n")
 
-             << wxT(" Snippets are accessed by using the context menu \"Edit\" \n")
-             << wxT(" or via the Properties context menu entry. \n")
-             << wxT("\n")
+             << wxT("Snippets are accessed by using the context menu \"Edit\" ")
+             << wxT("or via the Properties context menu entry.")
+             << wxT("\n\n")
 
-             << wxT(" Use the \"Settings\" menu to specify an external editor and \n")
-             << wxT(" to specify a non-default Snippets index file. \n")
-             << wxT("\n")
+             << wxT("Use the \"Settings\" menu to specify an external editor and ")
+             << wxT("to specify a non-default Snippets index file.")
+             << wxT("\n\n")
 
-             << wxT(" Both the text and file snippets may be dragged outward\n")
-             << wxT(" or copied to the clipboard.\n")
-             << wxT("\n")
+             << wxT("Both the text and file snippets may be dragged outward ")
+             << wxT("or copied to the clipboard.")
+             << wxT("\n\n")
 
-             << wxT(" Dragging a file snippet onto an external program window \n")
-             << wxT(" will open the file. Dragging it into the edit area will \n")
-             << wxT(" insert the text.\n");
-
-    GenericMessageBox( wxT("\n\n")+ buildInfo + helpText, _("About"),wxOK);
-
-    //wxMessageBox( wxT("\n\n") + buildInfo + wxT("\n\n") + helpText, _("About"), wxOK) ;
+             << wxT("Dragging a file snippet onto an external program window ")
+             << wxT("will open the file. Dragging it into the edit area will ")
+             << wxT("insert the text.");
 
+    wxMessageBox( wxT("\n\n")+ buildInfo + helpText, _("About"),wxOK);
 }
