@@ -1,10 +1,8 @@
 ///////////////////////////////////////////////////////////////////////////////
 // Name:        pdffontparsertruetype.cpp
-// Purpose:     
+// Purpose:
 // Author:      Ulrich Telle
-// Modified by:
 // Created:     2009-03-04
-// RCS-ID:      $$
 // Copyright:   (c) Ulrich Telle
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -25,6 +23,7 @@
 // includes
 #include <wx/filename.h>
 #include <wx/filesys.h>
+#include <wx/fontutil.h>
 #include <wx/zstream.h>
 
 #include "wx/pdffontdataopentype.h"
@@ -32,8 +31,6 @@
 #include "wx/pdffontparsertruetype.h"
 #include "wx/pdfencoding.h"
 
-#include "wxmemdbg.h"
-
 // --- Windows specific loading of TrueType font data
 
 #ifdef __WXMSW__
@@ -102,33 +99,33 @@ ExtractFontData(HDC hdc, DWORD& fontDataSize, BYTE*& fontData)
     BYTE uShortBuf[2];
     if (GetFontData(hdc, 0, 4, uShortBuf, 2) == GDI_ERROR)
     {
-      wxLogError(wxString(wxT("ExtractFontData: ")) +
+      wxLogError(wxString(wxS("ExtractFontData: ")) +
                  wxString::Format(_("Error %d on reading number of tables from TTC."), GetLastError()));
       return false;
     }
     nTables = ReadUShort(uShortBuf);
-    
+
     // 2. Calculate memory needed for the whole font header and read it into buffer
     DWORD headerSize = sizeOfFixedHeader + nTables * sizeOfTableEntry;
     BYTE* fontHeader = new BYTE[headerSize];
     if (!fontHeader)
     {
-      wxLogError(wxString(wxT("ExtractFontData: ")) +
+      wxLogError(wxString(wxS("ExtractFontData: ")) +
                  wxString(_("Out of memory while extracting from TTC.")));
       return false;
     }
     if (GetFontData(hdc, 0, 0, fontHeader, headerSize) == GDI_ERROR)
     {
       delete [] fontHeader;
-      wxLogError(wxString(wxT("ExtractFontData: ")) +
+      wxLogError(wxString(wxS("ExtractFontData: ")) +
                  wxString::Format(_("Error %d on reading font header from TTC."), GetLastError()));
       return false;
     }
-    
-    // 3. Calculate total font size. 
+
+    // 3. Calculate total font size.
     //    Tables are padded to 4-byte boundaries, so length should be rounded up to dword.
     DWORD bufferSize = headerSize;
-    USHORT i;    
+    USHORT i;
     for (i = 0; i < nTables; i++)
     {
       DWORD tableLength = ReadDWord(fontHeader + sizeOfFixedHeader + i * sizeOfTableEntry + offsetOfTableLength);
@@ -141,29 +138,29 @@ ExtractFontData(HDC hdc, DWORD& fontDataSize, BYTE*& fontData)
         bufferSize += tableLength;
       }
     }
-    
+
     // 4. Copying header into target buffer.
     //    Patch offsets with correct values while copying data.
     BYTE* buffer = new BYTE[bufferSize];
     if (buffer == NULL)
     {
       delete [] fontHeader;
-      wxLogError(wxString(wxT("ExtractFontData: ")) +
+      wxLogError(wxString(wxS("ExtractFontData: ")) +
                  wxString(_("Out of memory while extracting from TTC.")));
       return false;
     }
     memcpy(buffer, fontHeader, headerSize);
-    
-    // 5. Get table data from GDI, write it into known place 
+
+    // 5. Get table data from GDI, write it into known place
     //    inside target buffer and fix offset value.
     DWORD runningOffset = headerSize;
-    
+
     for (i = 0; i < nTables; i++)
     {
-      BYTE* entryData  = fontHeader + sizeOfFixedHeader + i * sizeOfTableEntry; 
+      BYTE* entryData  = fontHeader + sizeOfFixedHeader + i * sizeOfTableEntry;
       DWORD tableTag    = ReadTag(entryData + offsetOfTableTag);
       DWORD tableLength = ReadDWord(entryData + offsetOfTableLength);
-        
+
       // Write new offset for this table.
       WriteDWord(buffer + sizeOfFixedHeader + i * sizeOfTableEntry + offsetOfTableOffset, runningOffset);
 
@@ -172,7 +169,7 @@ ExtractFontData(HDC hdc, DWORD& fontDataSize, BYTE*& fontData)
       {
         delete [] buffer;
         delete [] fontHeader;
-        wxLogError(wxString(wxT("ExtractFontData: ")) +
+        wxLogError(wxString(wxS("ExtractFontData: ")) +
                    wxString::Format(_("Error %d on reading table data from TTC."), GetLastError()));
         return false;
       }
@@ -185,7 +182,7 @@ ExtractFontData(HDC hdc, DWORD& fontDataSize, BYTE*& fontData)
         {
           buffer[runningOffset] = 0;
           ++runningOffset;
-        } 
+        }
       }
     }
     delete [] fontHeader;
@@ -211,13 +208,13 @@ ExtractFontData(HDC hdc, DWORD& fontDataSize, BYTE*& fontData)
         else
         {
           delete [] buffer;
-          wxLogError(wxString(wxT("ExtractFontData: ")) +
+          wxLogError(wxString(wxS("ExtractFontData: ")) +
                      wxString::Format(_("Error %d on extracting font data."), GetLastError()));
         }
       }
       else
       {
-        wxLogError(wxString(wxT("ExtractFontData: ")) +
+        wxLogError(wxString(wxS("ExtractFontData: ")) +
                    wxString(_("Out of memory.")));
       }
     }
@@ -229,8 +226,7 @@ wxMemoryInputStream*
 wxPdfFontParserTrueType::LoadTrueTypeFontStream(const wxFont& font)
 {
   wxMemoryInputStream* fontStream = NULL;
-  LOGFONT lf;
-  wxFillLogFont(&lf, &font);
+  LOGFONT lf = font.GetNativeFontInfo()->lf;
 
   HDC hdc = CreateCompatibleDC(0);
   HFONT hFont = CreateFontIndirect(&lf);
@@ -332,7 +328,7 @@ static const wxChar codePages[] =
   /*  5 */ "1255 Hebrew",                  // wxFONTENCODING_CP1255
   /*  6 */ "1256 Arabic",                  // wxFONTENCODING_CP1256
   /*  7 */ "1257 Windows Baltic",          // wxFONTENCODING_CP1257
-  /*  8 */ "1258 Vietnamese",              // 
+  /*  8 */ "1258 Vietnamese",              //
   /*  9 */ null,
   /* 10 */ null,
   /* 11 */ null,
@@ -374,18 +370,18 @@ static const wxChar codePages[] =
   /* 15 */ null,
   /* 16 */ "869 IBM Greek",                   //
   /* 17 */ "866 MS-DOS Russian",              // wxFONTENCODING_CP866
-  /* 18 */ "865 MS-DOS Nordic",               // 
-  /* 19 */ "864 Arabic",                      // 
-  /* 20 */ "863 MS-DOS Canadian French",      // 
-  /* 21 */ "862 Hebrew",                      // 
-  /* 22 */ "861 MS-DOS Icelandic",            // 
-  /* 23 */ "860 MS-DOS Portuguese",           // 
-  /* 24 */ "857 IBM Turkish",                 // 
+  /* 18 */ "865 MS-DOS Nordic",               //
+  /* 19 */ "864 Arabic",                      //
+  /* 20 */ "863 MS-DOS Canadian French",      //
+  /* 21 */ "862 Hebrew",                      //
+  /* 22 */ "861 MS-DOS Icelandic",            //
+  /* 23 */ "860 MS-DOS Portuguese",           //
+  /* 24 */ "857 IBM Turkish",                 //
   /* 25 */ "855 IBM Cyrillic",                // wxFONTENCODING_CP855
   /* 26 */ "852 Latin 2",                     // wxFONTENCODING_CP852
-  /* 27 */ "775 MS-DOS Baltic",               // 
-  /* 28 */ "737 Greek; former 437 G",         // 
-  /* 29 */ "708 Arabic; ASMO 708",            // 
+  /* 27 */ "775 MS-DOS Baltic",               //
+  /* 28 */ "737 Greek; former 437 G",         //
+  /* 29 */ "708 Arabic; ASMO 708",            //
   /* 30 */ "850 WE/Latin 1",                  // wxFONTENCODING_CP850
   /* 31 */ "437 US"                           // wxFONTENCODING_CP437
 };
@@ -459,12 +455,6 @@ wxPdfFontParserTrueType::ClearTableDirectory()
   }
 }
 
-static const wxChar* tableNamesDefault[] = {
-  wxT("cvt "), wxT("fpgm"), wxT("glyf"), wxT("head"),
-  wxT("hhea"), wxT("hmtx"), wxT("loca"), wxT("maxp"), wxT("prep"),
-  NULL
-};
-
 void
 wxPdfFontParserTrueType::LockTable(const wxString& tableName)
 {
@@ -540,10 +530,10 @@ wxPdfFontParserTrueType::GetCollectionFontCount(const wxString& fontFileName)
     m_inFont->SeekI(0);
 
     // Check for TrueType collection
-    if (fileName.GetExt().Lower().IsSameAs(wxT("ttc")))
+    if (fileName.GetExt().Lower().IsSameAs(wxS("ttc")))
     {
       wxString mainTag = ReadString(4);
-      if (mainTag == wxT("ttcf"))
+      if (mainTag == wxS("ttcf"))
       {
         SkipBytes(4);
         count = ReadInt();
@@ -571,12 +561,12 @@ wxPdfFontParserTrueType::IdentifyFont(const wxString& fontFileName, int fontInde
     m_inFont->SeekI(0);
 
     // Check for TrueType collection
-    if (fileName.GetExt().Lower().IsSameAs(wxT("ttc")))
+    if (fileName.GetExt().Lower().IsSameAs(wxS("ttc")))
     {
       if (fontIndex >= 0)
       {
         wxString mainTag = ReadString(4);
-        if (mainTag == wxT("ttcf"))
+        if (mainTag == wxS("ttcf"))
         {
           SkipBytes(4);
           int dirCount = ReadInt();
@@ -587,21 +577,21 @@ wxPdfFontParserTrueType::IdentifyFont(const wxString& fontFileName, int fontInde
           }
           else
           {
-            wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+            wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                        wxString::Format(_("Font index %d out of range for font file '%s'."), fontIndex, fontFileName.c_str()));
             ok = false;
           }
         }
         else
         {
-          wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: '")) + 
+          wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: '")) +
                      wxString::Format(_("Font file '%s' not a valid TrueType collection (TTC) file."), fontFileName.c_str()));
           ok = false;
         }
       }
       else
       {
-        wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+        wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                    wxString::Format(_("Font index %d out of range for font file '%s'."), fontIndex, fontFileName.c_str()));
         ok = false;
       }
@@ -623,7 +613,7 @@ wxPdfFontParserTrueType::IdentifyFont(const wxString& fontFileName, int fontInde
       }
       else
       {
-        wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+        wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                    wxString::Format(_("Reading of font directory failed for font file '%s'."), fontFileName.c_str()));
       }
     }
@@ -631,7 +621,7 @@ wxPdfFontParserTrueType::IdentifyFont(const wxString& fontFileName, int fontInde
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                wxString::Format(_("Font file '%s' not accessible."), fontFileName.c_str()));
   }
   return fontData;
@@ -664,7 +654,7 @@ wxPdfFontParserTrueType::IdentifyFont(const wxFont& font)
       }
       else
       {
-        wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+        wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                    wxString::Format(_("Reading of font directory failed for font file '%s'."), font.GetFaceName().c_str()));
       }
     }
@@ -672,7 +662,7 @@ wxPdfFontParserTrueType::IdentifyFont(const wxFont& font)
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                wxString::Format(_("Font file '%s' not accessible."), font.GetFaceName().c_str()));
   }
   return fontData;
@@ -707,7 +697,7 @@ wxPdfFontParserTrueType::IdentifyFont(const wxFont& font)
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::IdentifyFont: ")) +
                wxString::Format(_("Reading of font directory failed for font file '%s'."), font.GetFaceName().c_str()));
   }
 #endif
@@ -813,7 +803,7 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
       {
         // Check whether the font file is a TrueType collection
         wxString mainTag = ReadString(4);
-        if (mainTag == wxT("ttcf"))
+        if (mainTag == wxS("ttcf"))
         {
           SkipBytes(4);
           int dirCount = ReadInt();
@@ -825,7 +815,7 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
           }
           else
           {
-            wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+            wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                        wxString::Format(_("Font index %d out of range for font file '%s'."), fontIndex, m_fileName.c_str()));
           }
         }
@@ -835,14 +825,14 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
           ok = (fontIndex == 0);
           if (!ok)
           {
-            wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: '")) + 
+            wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: '")) +
                        wxString::Format(_("Font file '%s' not a valid TrueType collection (TTC) file."), m_fileName.c_str()));
           }
         }
       }
       else
       {
-        wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+        wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                    wxString::Format(_("Font index %d out of range for font file '%s'."), fontIndex, m_fileName.c_str()));
       }
 
@@ -856,11 +846,11 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
             CheckCff();
             if (m_cff)
             {
-              ok = fontData->GetType().IsSameAs(wxT("OpenTypeUnicode"));
+              ok = fontData->GetType().IsSameAs(wxS("OpenTypeUnicode"));
             }
             else
             {
-              ok = fontData->GetType().IsSameAs(wxT("TrueTypeUnicode"));
+              ok = fontData->GetType().IsSameAs(wxS("TrueTypeUnicode"));
             }
             if (ok)
             {
@@ -868,19 +858,19 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
             }
             else
             {
-              wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+              wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                          wxString::Format(_("Wrong font data type '%s' for font file '%s'."), fontData->GetType().c_str(), m_fileName.c_str()));
             }
           }
           else
           {
-            wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+            wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                        wxString::Format(_("Missing font tables for font file '%s'."), m_fileName.c_str()));
           }
         }
         else
         {
-          wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+          wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                      wxString::Format(_("Reading of font directory failed for font file '%s'."), m_fileName.c_str()));
         }
       }
@@ -895,13 +885,13 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
     }
     else
     {
-      wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+      wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                  wxString::Format(_("Font file '%s' not accessible."), m_fileName.c_str()));
     }
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::LoadFontData: ")) +
                wxString(_("No font data instance given.")));
   }
   return ok;
@@ -925,7 +915,7 @@ wxPdfFontParserTrueType::PrepareFontData(wxPdfFontData* fontData)
       cMap = m_cmap10;
     else if (m_cmap31 != NULL)
       cMap = m_cmap31;
-    else 
+    else
       cMap = m_cmap10;
     wxPdfCMap::iterator cMapIter;
     int cc;
@@ -983,7 +973,7 @@ wxPdfFontParserTrueType::ReadTableDirectory()
     {
       if (!m_fileName.IsEmpty())
       {
-        wxLogError(wxString(wxT("wxPdfFontParserTrueType::ReadTableDirectory: '")) + 
+        wxLogError(wxString(wxS("wxPdfFontParserTrueType::ReadTableDirectory: '")) +
                    wxString::Format(_("Font file '%s' not a valid TrueType (TTF) or OpenType (OTF) file."), m_fileName.c_str()));
       }
       ok = false;
@@ -1026,9 +1016,9 @@ wxPdfFontParserTrueType::ReadTableDirectory()
 }
 
 static const wxChar* checkTableNames[] = {
-  wxT("cmap"), wxT("head"), wxT("hhea"), wxT("hmtx"), wxT("name"),
-  wxT("post"), 
-  wxT("glyf"), wxT("loca"),
+  wxS("cmap"), wxS("head"), wxS("hhea"), wxS("hmtx"), wxS("name"),
+  wxS("post"),
+  wxS("glyf"), wxS("loca"),
   NULL
 };
 
@@ -1036,7 +1026,7 @@ bool
 wxPdfFontParserTrueType::CheckTables()
 {
   bool ok = true;
-  int maxTableCount = (m_tableDirectory->find(wxT("CFF ")) == m_tableDirectory->end()) ? 8 : 6;
+  int maxTableCount = (m_tableDirectory->find(wxS("CFF ")) == m_tableDirectory->end()) ? 8 : 6;
   int tableCount = 0;
   while (ok && tableCount < maxTableCount && checkTableNames[tableCount] != NULL)
   {
@@ -1053,7 +1043,7 @@ void
 wxPdfFontParserTrueType::CheckCff()
 {
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("CFF "));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("CFF "));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
@@ -1073,11 +1063,11 @@ void
 wxPdfFontParserTrueType::CheckRestrictions()
 {
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("OS/2"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("OS/2"));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
-    LockTable(wxT("OS/2"));
+    LockTable(wxS("OS/2"));
     m_inFont->SeekI(tableLocation->m_offset+8);
     short fsType = ReadShort();
     bool rl = (fsType & 0x0002) != 0; // restricted license
@@ -1101,11 +1091,11 @@ wxPdfFontParserTrueType::GetBaseFont()
 {
   wxString fontName = wxEmptyString;
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("name"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("name"));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
-    LockTable(wxT("name"));
+    LockTable(wxS("name"));
     m_inFont->SeekI(tableLocation->m_offset+2);
     int numRecords = ReadUShort();
     int startOfStorage = ReadUShort();
@@ -1135,18 +1125,18 @@ wxPdfFontParserTrueType::GetBaseFont()
     if (fontName.IsEmpty())
     {
       wxFileName::SplitPath(m_fileName, NULL, &fontName, NULL);
-      fontName.Replace(wxT(" "), wxT("-"));
+      fontName.Replace(wxS(" "), wxS("-"));
     }
     ReleaseTable();
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::GetBaseFont: "))+
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::GetBaseFont: "))+
                wxString::Format(_("Table 'name' does not exist in font file '%s'."), m_fileName.c_str()));
   }
   return fontName;
 }
-    
+
 bool
 wxPdfFontParserTrueType::ReadMaps()
 {
@@ -1156,15 +1146,15 @@ wxPdfFontParserTrueType::ReadMaps()
   int realOS2TypoDescender;
 
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("head"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("head"));
   if (entry == m_tableDirectory->end())
   {
-    wxLogError(wxString(wxT("wxPdfFontParser::ReadMaps: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParser::ReadMaps: ")) +
                wxString::Format(_("Table 'head' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
     return false;
   }
   tableLocation = entry->second;
-  LockTable(wxT("head"));
+  LockTable(wxS("head"));
   m_inFont->SeekI(tableLocation->m_offset+16);
   head.m_flags = ReadUShort();
   head.m_unitsPerEm = ReadUShort();
@@ -1176,15 +1166,15 @@ wxPdfFontParserTrueType::ReadMaps()
   head.m_macStyle = ReadUShort();
   ReleaseTable();
 
-  entry = m_tableDirectory->find(wxT("hhea"));
+  entry = m_tableDirectory->find(wxS("hhea"));
   if (entry == m_tableDirectory->end())
   {
-    wxLogError(wxString(wxT("wxPdfFontParser::ReadMaps: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParser::ReadMaps: ")) +
                wxString::Format(_("Table 'hhea' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
     return false;
   }
   tableLocation = entry->second;
-  LockTable(wxT("hhea"));
+  LockTable(wxS("hhea"));
   m_inFont->SeekI(tableLocation->m_offset+4);
   hhea.m_ascender = ReadShort();
   hhea.m_descender = ReadShort();
@@ -1199,11 +1189,11 @@ wxPdfFontParserTrueType::ReadMaps()
   hhea.m_numberOfHMetrics = ReadUShort();
   ReleaseTable();
 
-  entry = m_tableDirectory->find(wxT("OS/2"));
+  entry = m_tableDirectory->find(wxS("OS/2"));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
-    LockTable(wxT("OS/2"));
+    LockTable(wxS("OS/2"));
     m_inFont->SeekI(tableLocation->m_offset);
     int version = ReadUShort();
     os_2.m_xAvgCharWidth = ReadShort();
@@ -1287,7 +1277,7 @@ wxPdfFontParserTrueType::ReadMaps()
   int underlineThickness = 50;
 
   double italicAngle;
-  entry = m_tableDirectory->find(wxT("post"));
+  entry = m_tableDirectory->find(wxS("post"));
   if (entry == m_tableDirectory->end())
   {
     static double pi = 4. * atan(1.0);
@@ -1298,7 +1288,7 @@ wxPdfFontParserTrueType::ReadMaps()
   else
   {
     tableLocation = entry->second;
-    LockTable(wxT("post"));
+    LockTable(wxS("post"));
     m_inFont->SeekI(tableLocation->m_offset+4);
     short mantissa = ReadShort();
     int fraction = ReadUShort();
@@ -1341,22 +1331,22 @@ wxPdfFontParserTrueType::ReadMaps()
   }
   m_fd.SetFlags(flags);
 
-  wxString fbb = wxString::Format(wxT("[%d %d %d %d]"),
+  wxString fbb = wxString::Format(wxS("[%d %d %d %d]"),
                    (int) (head.m_xMin * 1000 / head.m_unitsPerEm),
                    (int) (head.m_yMin * 1000 / head.m_unitsPerEm),
                    (int) (head.m_xMax * 1000 / head.m_unitsPerEm),
                    (int) (head.m_yMax * 1000 / head.m_unitsPerEm));
   m_fd.SetFontBBox(fbb);
 
-  entry = m_tableDirectory->find(wxT("cmap"));
+  entry = m_tableDirectory->find(wxS("cmap"));
   if (entry == m_tableDirectory->end())
   {
-    wxLogError(wxString(wxT("wxPdfFontParser::ReadMaps: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParser::ReadMaps: ")) +
                wxString::Format(_("Table 'cmap' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
     return false;
   }
   tableLocation = entry->second;
-  LockTable(wxT("cmap"));
+  LockTable(wxS("cmap"));
   m_inFont->SeekI(tableLocation->m_offset);
   SkipBytes(2);
   int num_tables = ReadUShort();
@@ -1465,7 +1455,7 @@ wxPdfFontParserTrueType::ReadMaps()
   bool ok = (m_cmap10 != NULL) || (m_cmap31 != NULL) || (m_cmapExt != NULL);
   if (!ok)
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::ReadMaps: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::ReadMaps: ")) +
                wxString::Format(_("No valid 'cmap' table found for font '%s'."),  m_fontName.c_str()));
   }
 
@@ -1476,15 +1466,15 @@ bool
 wxPdfFontParserTrueType::ReadGlyphWidths(int numberOfHMetrics, int unitsPerEm)
 {
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("hmtx"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("hmtx"));
   if (entry == m_tableDirectory->end())
   {
-    wxLogError(wxString(wxT("wxPdfFontParser::ReadGlyphWidths: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParser::ReadGlyphWidths: ")) +
                wxString::Format(_("Table 'hmtx' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
     return false;
   }
   tableLocation = entry->second;
-  LockTable(wxT("hmtx"));
+  LockTable(wxS("hmtx"));
   m_inFont->SeekI(tableLocation->m_offset);
 
   m_glyphWidths.SetCount(numberOfHMetrics);
@@ -1513,7 +1503,7 @@ wxPdfFontParserTrueType::ReadFormat0()
   }
   return h;
 }
-    
+
 wxPdfCMap*
 wxPdfFontParserTrueType::ReadFormat4()
 {
@@ -1574,7 +1564,7 @@ wxPdfFontParserTrueType::ReadFormat4()
       r->m_width = GetGlyphWidth(r->m_glyph);
       int idx = m_fontSpecific ? ((j & 0xff00) == 0xf000 ? j & 0xff : j) : j;
       (*h)[idx] = r;
-//      wxLogMessage(wxT("C %ld G %ld"), idx, glyph);
+//      wxLogMessage(wxS("C %ld G %ld"), idx, glyph);
     }
   }
 
@@ -1586,7 +1576,7 @@ wxPdfFontParserTrueType::ReadFormat4()
 
   return h;
 }
-    
+
 wxPdfCMap*
 wxPdfFontParserTrueType::ReadFormat6()
 {
@@ -1636,11 +1626,11 @@ void
 wxPdfFontParserTrueType::ReadKerning(int unitsPerEm)
 {
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("kern"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("kern"));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
-    LockTable(wxT("kern"));
+    LockTable(wxS("kern"));
     m_kp = new wxPdfKernPairMap();
     wxPdfKernWidthMap* kwMap = NULL;
     wxPdfKernWidthMap::iterator kw;
@@ -1689,7 +1679,7 @@ wxPdfFontParserTrueType::ReadKerning(int unitsPerEm)
     ReleaseTable();
   }
 }
-    
+
 int
 wxPdfFontParserTrueType::GetGlyphWidth(unsigned int glyph)
 {
@@ -1721,11 +1711,11 @@ wxPdfFontParserTrueType::GetNames(int id, bool namesOnly)
 {
   wxArrayString names;
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("name"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("name"));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
-    LockTable(wxT("name"));
+    LockTable(wxS("name"));
     m_inFont->SeekI(tableLocation->m_offset+2);
     int numRecords = ReadUShort();
     int startOfStorage = ReadUShort();
@@ -1753,9 +1743,9 @@ wxPdfFontParserTrueType::GetNames(int id, bool namesOnly)
         }
         if (!namesOnly)
         {
-          names.Add(wxString::Format(wxT("%d"), platformID));
-          names.Add(wxString::Format(wxT("%d"), platformEncodingID));
-          names.Add(wxString::Format(wxT("%d"), languageID));
+          names.Add(wxString::Format(wxS("%d"), platformID));
+          names.Add(wxString::Format(wxS("%d"), platformEncodingID));
+          names.Add(wxString::Format(wxS("%d"), languageID));
         }
         names.Add(name);
         m_inFont->SeekI(pos);
@@ -1765,7 +1755,7 @@ wxPdfFontParserTrueType::GetNames(int id, bool namesOnly)
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParser::GetNames: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParser::GetNames: ")) +
                wxString::Format(_("Table 'name' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
   }
   return names;
@@ -1776,11 +1766,11 @@ wxPdfFontParserTrueType::GetEnglishName(int id)
 {
   wxString englishName = wxEmptyString;
   wxPdfTableDirectoryEntry* tableLocation;
-  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxT("name"));
+  wxPdfTableDirectory::iterator entry = m_tableDirectory->find(wxS("name"));
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
-    LockTable(wxT("name"));
+    LockTable(wxS("name"));
     m_inFont->SeekI(tableLocation->m_offset+2);
     int numRecords = ReadUShort();
     int startOfStorage = ReadUShort();
@@ -1826,7 +1816,7 @@ wxPdfFontParserTrueType::GetEnglishName(int id)
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontParserTrueType::GetEnglishName: ")) +
+    wxLogError(wxString(wxS("wxPdfFontParserTrueType::GetEnglishName: ")) +
                wxString::Format(_("Table 'name' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
   }
   return englishName;
