@@ -2,7 +2,6 @@
 // Name:        pdfimage.cpp
 // Purpose:     Implementation of wxPdfImage classes
 // Author:      Ulrich Telle
-// Modified by:
 // Created:     2005-08-11
 // Copyright:   (c) Ulrich Telle
 // Licence:     wxWindows licence
@@ -60,10 +59,10 @@ wxPdfImage::wxPdfImage(wxPdfDocument* document, int index, const wxString& filen
 
   m_width    = 0;
   m_height   = 0;
-  m_cs       = wxT("");
+  m_cs       = wxS("");
   m_bpc      = '\0';
-  m_f        = wxT("");
-  m_parms    = wxT("");
+  m_f        = wxS("");
+  m_parms    = wxS("");
 
   m_palSize  = 0;
   m_pal      = NULL;
@@ -104,10 +103,10 @@ wxPdfImage::wxPdfImage(wxPdfDocument* document, int index, const wxString& name,
 
   m_width    = 0;
   m_height   = 0;
-  m_cs       = wxT("");
+  m_cs       = wxS("");
   m_bpc      = '\0';
-  m_f        = wxT("");
-  m_parms    = wxT("");
+  m_f        = wxS("");
+  m_parms    = wxS("");
 
   m_palSize  = 0;
   m_pal      = NULL;
@@ -134,10 +133,10 @@ wxPdfImage::wxPdfImage(wxPdfDocument* document, int index, const wxString& name,
 
   m_width    = 0;
   m_height   = 0;
-  m_cs       = wxT("");
+  m_cs       = wxS("");
   m_bpc      = '\0';
-  m_f        = wxT("");
-  m_parms    = wxT("");
+  m_f        = wxS("");
+  m_parms    = wxS("");
 
   m_palSize  = 0;
   m_pal      = NULL;
@@ -191,13 +190,13 @@ wxPdfImage::ConvertWxImage(const wxImage& image, bool jpegFormat)
 #if wxUSE_LIBJPEG
     if (jpegFormat)
     {
-      m_type = wxT("jpeg");
+      m_type = wxS("jpeg");
       isValid = ParseJPG(&is);
     }
     else
 #endif // wxUSE_LIBJPEG
     {
-      m_type = wxT("png");
+      m_type = wxS("png");
       isValid = ParsePNG(&is);
     }
   }
@@ -214,27 +213,27 @@ wxPdfImage::Parse()
 
   if (m_imageStream)
   {
-    if ((m_type.StartsWith(wxT("image/")) && m_type.EndsWith(wxT("png"))) ||
-        m_type == wxT("png"))
+    if ((m_type.StartsWith(wxS("image/")) && m_type.EndsWith(wxS("png"))) ||
+        m_type == wxS("png"))
     {
       isValid = ParsePNG(m_imageStream);
     }
-    else if ((m_type.StartsWith(wxT("image/")) && m_type.EndsWith(wxT("jpeg"))) ||
-             m_type == wxT("jpeg") || m_type == wxT("jpg"))
+    else if ((m_type.StartsWith(wxS("image/")) && m_type.EndsWith(wxS("jpeg"))) ||
+             m_type == wxS("jpeg") || m_type == wxS("jpg"))
     {
       isValid = ParseJPG(m_imageStream);
     }
 #if wxUSE_GIF
-    else if ((m_type.StartsWith(wxT("image/")) && m_type.EndsWith(wxT("gif"))) ||
-             m_type == wxT("gif"))
+    else if ((m_type.StartsWith(wxS("image/")) && m_type.EndsWith(wxS("gif"))) ||
+             m_type == wxS("gif"))
     {
       isValid = ParseGIF(m_imageStream);
     }
 #endif // wxUSE_GIF
     else
     {
-      if ((m_type.StartsWith(wxT("image/")) && m_type.EndsWith(wxT("wmf"))) ||
-          m_type == wxT("wmf") || m_name.Right(2) == wxT(".wmf"))
+      if ((m_type.StartsWith(wxS("image/")) && m_type.EndsWith(wxS("wmf"))) ||
+          m_type == wxS("wmf") || m_name.Right(2) == wxS(".wmf"))
       {
         m_isFormObj = true;
         isValid = ParseWMF(m_imageStream);
@@ -262,7 +261,7 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   if (strncmp(buffer,"\x89PNG\x0D\x0A\x1A\x0A",8) != 0)
   {
     // Not a PNG file
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("'%s' not a PNG file."), m_name.c_str()));
     return false;
   }
@@ -273,7 +272,7 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   if (strncmp(buffer,"IHDR",4) != 0)
   {
     // Incorrect PNG file
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("Incorrect PNG file '%s'."), m_name.c_str()));
     return false;
   }
@@ -286,7 +285,7 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   if (bpc > 8)
   {
     // 16-bit depth not supported
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("16-bit depth not supported: '%s'."), m_name.c_str()));
     return false;
   }
@@ -296,20 +295,20 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   char ct = buffer[0];
   if (ct == 0)
   {
-    colspace = wxT("DeviceGray");
+    colspace = wxS("DeviceGray");
   }
   else if (ct == 2)
   {
-    colspace = wxT("DeviceRGB");
+    colspace = wxS("DeviceRGB");
   }
   else if (ct == 3)
   {
-    colspace = wxT("Indexed");
+    colspace = wxS("Indexed");
   }
   else
   {
     // Unknown colour type
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("Unknown colour type: '%s'."), m_name.c_str()));
     return false;
   }
@@ -318,27 +317,27 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   if (buffer[0] != 0)
   {
     // Unknown compression method
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("Unknown compression method: '%s'."), m_name.c_str()));
     return false;
   }
   if (buffer[1] != 0)
   {
     // Unknown filter method
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("Unknown filter method: '%s'."), m_name.c_str()));
     return false;
   }
   if (buffer[2] != 0)
   {
     // Interlacing not supported
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("Interlacing not supported: '%s'."), m_name.c_str()));
     return false;
   }
 
   imageStream->Read(buffer,4);
-  m_parms = wxString::Format(wxT("/DecodeParms <</Predictor 15 /Colors %d /BitsPerComponent %d /Columns %d>>"), (ct==2 ? 3 : 1), (int) bpc, w);
+  m_parms = wxString::Format(wxS("/DecodeParms <</Predictor 15 /Colors %d /BitsPerComponent %d /Columns %d>>"), (ct==2 ? 3 : 1), (int) bpc, w);
 
   // Scan chunks looking for palette, transparency and image data
   m_palSize  = 0;
@@ -424,13 +423,13 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   }
   while (n);
 
-  if (colspace == wxT("Indexed") && m_pal == NULL)
+  if (colspace == wxS("Indexed") && m_pal == NULL)
   {
     if (m_pal  != NULL) delete [] m_pal;
     if (m_trns != NULL) delete [] m_trns;
     if (m_data != NULL) delete [] m_data;
     // Missing palette
-    wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParsePNG: ")) +
                wxString::Format(_("Missing palette: '%s'."), m_name.c_str()));
     return false;
   }
@@ -439,7 +438,7 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   m_height = h;
   m_cs     = colspace;
   m_bpc    = bpc;
-  m_f      = wxT("FlateDecode");
+  m_f      = wxS("FlateDecode");
 
   isValid = true;
   return isValid;
@@ -473,7 +472,7 @@ bool
 wxPdfImage::ParseJPG(wxInputStream* imageStream)
 {
   bool isValid = false;
-  wxString colspace = wxT("");
+  wxString colspace = wxS("");
 
   m_palSize  = 0;
   m_pal      = NULL;
@@ -487,16 +486,14 @@ wxPdfImage::ParseJPG(wxInputStream* imageStream)
   if (strncmp((const char*) buffer,"\xff\xd8\xff",3) != 0)
   {
     // Not a JPEG file
-    wxLogDebug(wxString(wxT("wxPdfImage::ParseJPG: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParseJPG: ")) +
                wxString::Format(_("'%s' not a JPEG file."), m_name.c_str()));
     return false;
   }
 
   // Extract info from a JPEG file
   unsigned int   marker = M_PSEUDO;
-/* C::B begin */
   unsigned short ffRead = 1;
-/* C::B end */
   unsigned char  bits     = 0;
   unsigned short height   = 0;
   unsigned short width    = 0;
@@ -587,9 +584,7 @@ wxPdfImage::ParseJPG(wxInputStream* imageStream)
       case M_SOF14:
       case M_SOF15:
         // handle SOFn block
-/* C::B begin */
-        ReadUShortBE(imageStream);
-/* C::B end */
+        imageStream->SeekI(2, wxFromCurrent); // skip 2-byte length field
         imageStream->Read(&bits,1);
         height = ReadUShortBE(imageStream);
         width  = ReadUShortBE(imageStream);
@@ -621,15 +616,15 @@ wxPdfImage::ParseJPG(wxInputStream* imageStream)
   {
     if (channels == 3)
     {
-      colspace = wxT("DeviceRGB");
+      colspace = wxS("DeviceRGB");
     }
     else if(channels == 4)
     {
-      colspace = wxT("DeviceCMYK");
+      colspace = wxS("DeviceCMYK");
     }
     else
     {
-      colspace = wxT("DeviceGray");
+      colspace = wxS("DeviceGray");
     }
     m_bpc = bits;
 
@@ -643,7 +638,7 @@ wxPdfImage::ParseJPG(wxInputStream* imageStream)
     m_height = height;
     m_cs     = colspace;
     m_bpc    = bits;
-    m_f      = wxT("DCTDecode");
+    m_f      = wxS("DCTDecode");
   }
 
   return isValid;
@@ -668,14 +663,14 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   wxGIFDecoder gif;
   if (!gif.CanRead(*imageStream))
   {
-    wxLogDebug(wxString(wxT("wxPdfImage::ParseGIF: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParseGIF: ")) +
                wxString::Format(_("'%s' not a GIF file."), m_name.c_str()));
     return false;
   }
 
   if (gif.LoadGIF(*imageStream) != wxGIF_OK)
   {
-    wxLogDebug(wxString(wxT("wxPdfImage::ParseGIF: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParseGIF: ")) +
                wxString::Format(_("Invalid GIF file '%s'."), m_name.c_str()));
     return false;
   }
@@ -684,7 +679,7 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   wxSize gifSize = gif.GetFrameSize(0);
   m_width = gifSize.GetWidth();
   m_height = gifSize.GetHeight();
-  m_cs = wxT("Indexed");
+  m_cs = wxS("Indexed");
   m_bpc    = 8;
 
   m_palSize  = 768;
@@ -704,7 +699,7 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   m_dataSize = m_width * m_height;
   if (m_document->m_compress)
   {
-    m_f = wxT("FlateDecode");
+    m_f = wxS("FlateDecode");
     wxMemoryOutputStream* p = new wxMemoryOutputStream();
     wxZlibOutputStream q(*p);
     q.Write(gif.GetData(0),m_dataSize);
@@ -716,7 +711,7 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   }
   else
   {
-    m_f = wxT("");
+    m_f = wxS("");
     m_data = new char[m_dataSize];
     memcpy(m_data,gif.GetData(0),m_dataSize);
   }
@@ -724,14 +719,14 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   wxGIFDecoder gif(imageStream);
   if (!gif.CanRead())
   {
-    wxLogDebug(wxString(wxT("wxPdfImage::ParseGIF: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParseGIF: ")) +
                wxString::Format(_("'%s' not a GIF file."), m_name.c_str()));
     return false;
   }
 
   if (gif.ReadGIF() != wxGIF_OK)
   {
-    wxLogDebug(wxString(wxT("wxPdfImage::ParseGIF: ")) +
+    wxLogDebug(wxString(wxS("wxPdfImage::ParseGIF: ")) +
                wxString::Format(_("Invalid GIF file '%s'."), m_name.c_str()));
     return false;
   }
@@ -739,7 +734,7 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   isValid = true;
   m_width = gif.GetWidth();
   m_height = gif.GetHeight();
-  m_cs = wxT("Indexed");
+  m_cs = wxS("Indexed");
   m_bpc    = 8;
 
   m_palSize  = 768;
@@ -759,7 +754,7 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   m_dataSize = m_width * m_height;
   if (m_document->m_compress)
   {
-    m_f = wxT("FlateDecode");
+    m_f = wxS("FlateDecode");
     wxMemoryOutputStream* p = new wxMemoryOutputStream();
     wxZlibOutputStream q(*p);
     q.Write(gif.GetData(),m_dataSize);
@@ -771,7 +766,7 @@ wxPdfImage::ParseGIF(wxInputStream* imageStream)
   }
   else
   {
-    m_f = wxT("");
+    m_f = wxS("");
     m_data = new char[m_dataSize];
     memcpy(m_data,gif.GetData(),m_dataSize);
   }
@@ -971,9 +966,9 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
               }
               else
               {
-                data += wxPdfUtility::Double2String(obj->r/255.,3) + wxString(wxT(" "));
-                data += wxPdfUtility::Double2String(obj->g/255.,3) + wxString(wxT(" "));
-                data += wxPdfUtility::Double2String(obj->b/255.,3) + wxString(wxT(" rg\n"));
+                data += wxPdfUtility::Double2String(obj->r/255.,3) + wxString(wxS(" "));
+                data += wxPdfUtility::Double2String(obj->g/255.,3) + wxString(wxS(" "));
+                data += wxPdfUtility::Double2String(obj->b/255.,3) + wxString(wxS(" rg\n"));
               }
               break;
 
@@ -1024,25 +1019,25 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
 
               if (!nullPen)
               {
-                data += wxPdfUtility::Double2String(obj->r/255.,3) + wxString(wxT(" "));
-                data += wxPdfUtility::Double2String(obj->g/255.,3) + wxString(wxT(" "));
-                data += wxPdfUtility::Double2String(obj->b/255.,3) + wxString(wxT(" RG\n"));
+                data += wxPdfUtility::Double2String(obj->r/255.,3) + wxString(wxS(" "));
+                data += wxPdfUtility::Double2String(obj->g/255.,3) + wxString(wxS(" "));
+                data += wxPdfUtility::Double2String(obj->b/255.,3) + wxString(wxS(" RG\n"));
 
-                data += wxPdfUtility::Double2String(obj->width*m_document->m_k,2) + wxString(wxT(" w\n"));
+                data += wxPdfUtility::Double2String(obj->width*m_document->m_k,2) + wxString(wxS(" w\n"));
               }
 
               if (lenDashArray > 0)
               {
-                wxString s = wxT("[");
+                wxString s = wxS("[");
                 for (i = 0; i < lenDashArray; i++)
                 {
                   s += wxPdfUtility::Double2String(dashArray[i] * m_document->m_k,4);
                   if (i != lenDashArray-1)
                   {
-                    s += wxT(" ");
+                    s += wxS(" ");
                   }
                 }
-                s += wxT("] 0 d\n");
+                s += wxS("] 0 d\n");
                 data += s;
               }
               break;
@@ -1067,17 +1062,17 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
 
             if (k < numpoints)
             {
-              data += wxString::Format(wxT("%d %d l\n"), (int) px, (int) py);
+              data += wxString::Format(wxS("%d %d l\n"), (int) px, (int) py);
             }
             else
             {
-              data += wxString::Format(wxT("%d %d m\n"), (int) px, (int) py);
+              data += wxString::Format(wxS("%d %d m\n"), (int) px, (int) py);
             }
           }
 
           if (func == 0x0325)
           {
-            op = wxT("s");
+            op = wxS("s");
           }
           else if (func == 0x0324)
           {
@@ -1085,31 +1080,31 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
             {
               if (nullBrush)
               {
-                op = wxT("n");  // no op
+                op = wxS("n");  // no op
               }
               else
               {
-                op = wxT("f");  // fill
+                op = wxS("f");  // fill
               }
             }
             else
             {
               if (nullBrush)
               {
-                op = wxT("s");  // stroke
+                op = wxS("s");  // stroke
               }
               else
               {
-                op = wxT("b");  // stroke and fill
+                op = wxS("b");  // stroke and fill
               }
             }
 
-            if (polyFillMode == 1 && (op == wxT("b") || op == wxT("f")))
+            if (polyFillMode == 1 && (op == wxS("b") || op == wxS("f")))
             {
-              op += wxT("*");  // use even-odd fill rule
+              op += wxS("*");  // use even-odd fill rule
             }
           }
-          data += op + wxT("\n");
+          data += op + wxS("\n");
           delete [] coords;
         }
         break;
@@ -1136,11 +1131,11 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
 
               if (k == numpoints)
               {
-                data += wxString::Format(wxT("%d %d m\n"), (int) px, (int) py);
+                data += wxString::Format(wxS("%d %d m\n"), (int) px, (int) py);
               }
               else
               {
-                data += wxString::Format(wxT("%d %d m\n"), (int) px, (int) py);
+                data += wxString::Format(wxS("%d %d m\n"), (int) px, (int) py);
               }
             }
 
@@ -1151,31 +1146,31 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
           {
             if (nullBrush)
             {
-              op = wxT("n");  // no op
+              op = wxS("n");  // no op
             }
             else
             {
-              op = wxT("f");  // fill
+              op = wxS("f");  // fill
             }
           }
           else
           {
             if (nullBrush)
             {
-              op = wxT("s");  // stroke
+              op = wxS("s");  // stroke
             }
             else
             {
-              op = wxT("b");  // stroke and fill
+              op = wxS("b");  // stroke and fill
             }
           }
 
-          if (polyFillMode == 1 && (op == wxT("b") || op == wxT("f")))
+          if (polyFillMode == 1 && (op == wxS("b") || op == wxS("f")))
           {
-            op += wxT("*");  // use even-odd fill rule
+            op += wxS("*");  // use even-odd fill rule
           }
 
-          data += op + wxT("\n");
+          data += op + wxS("\n");
           delete [] coords;
         }
         break;
