@@ -28,6 +28,8 @@
 #include <logmanager.h>
 #include <macrosmanager.h>
 
+#include <cinttypes>
+
 #include "debugger_defs.h"
 #include "debuggergdb.h"
 #include "gdb_driver.h"
@@ -181,8 +183,6 @@ static wxRegEx reInfoProgramProcess(_T("child process ([0-9]+)"));
 static wxRegEx reInfoThreads(_T("(\\**)[ \t]*([0-9]+)[ \t](.*)"));
 static wxRegEx reGenericHexAddress(_T("(0x[A-Fa-f0-9]+)"));
 
-static wxRegEx reExamineMemoryLine(wxT("[ \t]*(0x[0-9a-f]+)[ \t]<.+>:[ \t]+(.+)"));
-
 //mi output from 'nexti' is:
 //"^Z^Zc:\dev\cb_exp\cb_debugviz\panels.cpp:409:12533:middle:0x4044f5"
 //That's tough - guessing that path may/not always be full, i.e. might be
@@ -343,6 +343,7 @@ class GdbCmd_AttachToProcess : public DebuggerCmd
                     // log this and quit debugging
                     m_pDriver->Log(_("Attaching failed: ")+lines[i]);
                     m_pDriver->QueueCommand(new DebuggerCmd(m_pDriver, _T("quit")));
+                    m_pDriver->MarkProgramStopped(true);
                 }
 //                m_pDriver->DebugLog(lines[i]);
             }
@@ -803,17 +804,32 @@ class GdbCmd_Watch : public DebuggerCmd
                 default:            break;
             }
 
-            // auto-set array types
-            if (!m_watch->IsArray() &&  m_watch->GetFormat() == Undefined && type.Contains(_T('[')))
-                m_watch->SetArray(true);
-
-            if (m_watch->IsArray() && m_watch->GetArrayCount() > 0)
+            if (g_DebugLanguage == dl_Cpp)
             {
-                m_Cmd << wxT("(") << symbol << wxT(")");
-                m_Cmd << wxString::Format(_T("[%d]@%d"), m_watch->GetArrayStart(), m_watch->GetArrayCount());
+                // auto-set array types
+                if (!m_watch->IsArray() &&  m_watch->GetFormat() == Undefined && type.Contains(_T('[')))
+                    m_watch->SetArray(true);
+
+                if (m_watch->IsArray() && m_watch->GetArrayCount() > 0)
+                {
+                    m_Cmd << wxT("(") << symbol << wxT(")");
+                    m_Cmd << wxString::Format(_T("[%d]@%d"), m_watch->GetArrayStart(), m_watch->GetArrayCount());
+                }
+                else
+                    m_Cmd << symbol;
+            }
+            else  // (g_DebugLanguage == dl_Fortran)
+            {
+                if (m_watch->IsArray() && m_watch->GetArrayCount() > 0)
+                {
+                    if (m_watch->GetArrayStart() < 1)
+                        m_watch->SetArrayParams(1, m_watch->GetArrayCount());
+                    m_Cmd << symbol;
+                    m_Cmd << wxString::Format(_T("(%d)@%d"), m_watch->GetArrayStart(), m_watch->GetArrayCount());
+                }
+                else
+                    m_Cmd << symbol;
             }
-            else
-                m_Cmd << symbol;
         }
         void ParseOutput(const wxString& output)
         {
@@ -832,6 +848,52 @@ class GdbCmd_Watch : public DebuggerCmd
         }
 };
 
+
+/**
+  * Command to read a X bytes from a specific address.
+  */
+class GdbCmd_MemoryRangeWatch : public DebuggerCmd
+{
+        cb::shared_ptr<GDBMemoryRangeWatch> m_watch;
+        wxString m_ParseFunc;
+    public:
+        GdbCmd_MemoryRangeWatch(DebuggerDriver* driver, cb::shared_ptr<GDBMemoryRangeWatch> watch) :
+            DebuggerCmd(driver),
+            m_watch(watch)
+        {
+            // wx2.8 does not support uint64_z for wxstring::printf
+            const size_t tmpBuffSize = 20;
+            char tmpAddr[tmpBuffSize];
+            char tmpSize[tmpBuffSize];
+            memset(tmpAddr, 0, tmpBuffSize);
+            memset(tmpSize, 0, tmpBuffSize);
+            snprintf(tmpAddr, tmpBuffSize, "0x%" PRIx64, m_watch->GetAddress());
+            snprintf(tmpSize, tmpBuffSize, "%" PRIu64, m_watch->GetSize());
+
+            m_Cmd  = wxString(wxT("x /")) << wxString::FromUTF8(tmpSize) << wxT("xb ") << wxString::FromUTF8(tmpAddr);
+        }
+
+        void ParseOutput(const wxString& output)
+        {
+            wxArrayString lines = GetArrayFromString(output, _T('\n'));
+            wxString addr;
+            std::vector<uint8_t> memory, lineMemory;
+            for (unsigned int i = 0; i < lines.GetCount(); ++i)
+            {
+                lineMemory.clear();
+                // TODO: handle errors here!
+                ParseGDBExamineMemoryLine(addr, lineMemory, lines[i]);
+
+                memory.insert(memory.end(), lineMemory.begin(), lineMemory.end());
+            }
+
+            wxString watchString;
+            watchString = wxString::From8BitData(reinterpret_cast<const char*>(memory.data()),
+                                                 memory.size());
+            m_watch->SetValue(watchString);
+        }
+};
+
 /**
   * Command to get a watched variable's type.
   */
@@ -1494,7 +1556,7 @@ class GdbCmd_DisassemblyInit : public DebuggerCmd
             else
                 m_Cmd << _T("disassemble ") << m_hexAddrStr << _T("\n") ;
 
-            m_Cmd << _T("info frame\n") << _T("end");
+            m_Cmd << _T("info frame\n") << _T("end\n");
         };
 
         void ParseOutput(const wxString& p_output)
@@ -1592,47 +1654,32 @@ class GdbCmd_ExamineMemory : public DebuggerCmd
         }
         void ParseOutput(const wxString& output)
         {
-            // output is a series of:
-            //
-            // 0x22ffc0:       0xf0    0xff    0x22    0x00    0x4f    0x6d    0x81    0x7c
-            // or
-            // 0x85267a0 <RS485TxTask::taskProc()::rcptBuf>:   0x00   0x00   0x00   0x00   0x00   0x00   0x00   0x00
-
             cbExamineMemoryDlg *dialog = Manager::Get()->GetDebuggerManager()->GetExamineMemoryDialog();
 
             dialog->Begin();
             dialog->Clear();
 
+            wxString addr;
+            std::vector<uint8_t> values;
             wxArrayString lines = GetArrayFromString(output, _T('\n'));
-            wxString addr, memory;
             for (unsigned int i = 0; i < lines.GetCount(); ++i)
             {
-                if (reExamineMemoryLine.Matches(lines[i]))
+                if (ParseGDBExamineMemoryLine(addr, values, lines[i]))
                 {
-                    addr = reExamineMemoryLine.GetMatch(lines[i], 1);
-                    memory = reExamineMemoryLine.GetMatch(lines[i], 2);
+                    wxString hexByte;
+                    for (uint8_t v : values)
+                    {
+                        hexByte = wxString::Format(wxT("%02x"), v);
+                        dialog->AddHexByte(addr, hexByte);
+                    }
                 }
                 else
                 {
-                    if (lines[i].First(_T(':')) == -1)
-                    {
-                        dialog->AddError(lines[i]);
-                        continue;
-                    }
-                    addr = lines[i].BeforeFirst(_T(':'));
-                    memory = lines[i].AfterFirst(_T(':'));
-                }
-
-                size_t pos = memory.find(_T('x'));
-                while (pos != wxString::npos)
-                {
-                    wxString hexbyte;
-                    hexbyte << memory[pos + 1];
-                    hexbyte << memory[pos + 2];
-                    dialog->AddHexByte(addr, hexbyte);
-                    pos = memory.find(_T('x'), pos + 1); // skip current 'x'
+                    dialog->AddError(lines[i]);
+                    continue;
                 }
             }
+
             dialog->End();
         }
 };
@@ -1654,7 +1701,7 @@ class GdbCmd_RemoteBaud : public DebuggerCmd
 class GdbCmd_RemoteTarget : public DebuggerCmd
 {
     public:
-        GdbCmd_RemoteTarget(DebuggerDriver* driver, RemoteDebugging* rd)
+        GdbCmd_RemoteTarget(DebuggerDriver* driver, const RemoteDebugging *rd)
             : DebuggerCmd(driver)
         {
             const wxString targetRemote = rd->extendedRemote ? _T("target extended-remote ") : _T("target remote ");
@@ -1856,4 +1903,25 @@ class GdbCmd_FindCursor : public DebuggerCmd
         }
 };
 
+/**
+  * Command to determine the debugging (working) language.
+  */
+class GdbCmd_DebugLanguage : public DebuggerCmd
+{
+    public:
+        GdbCmd_DebugLanguage(DebuggerDriver* driver)
+            : DebuggerCmd(driver)
+        {
+            m_Cmd << _T("show language");
+        }
+
+        void ParseOutput(const wxString& output)
+        {
+            if (output.Lower().Find(wxT("fortran")) != wxNOT_FOUND)
+                g_DebugLanguage = dl_Fortran;
+            else
+                g_DebugLanguage = dl_Cpp;
+        }
+};
+
 #endif // DEBUGGER_COMMANDS_H
