@@ -2,9 +2,9 @@
  * This file is part of the Code::Blocks IDE and licensed under the GNU General Public License, version 3
  * http://www.gnu.org/licenses/gpl-3.0.html
  *
- * $Revision: 11082 $
- * $Id: parsewatchvalue.cpp 11082 2017-06-04 12:24:01Z fuscated $
- * $HeadURL: http://svn.code.sf.net/p/codeblocks/code/branches/release-17.xx/src/plugins/debuggergdb/parsewatchvalue.cpp $
+ * $Revision: 11633 $
+ * $Id: parsewatchvalue.cpp 11633 2019-04-20 16:56:13Z fuscated $
+ * $HeadURL: svn://svn.code.sf.net/p/codeblocks/code/branches/release-20.xx/src/plugins/debuggergdb/parsewatchvalue.cpp $
  */
 
 #include <sdk.h>
@@ -333,12 +333,21 @@ inline cb::shared_ptr<GDBWatch> AddChild(cb::shared_ptr<GDBWatch> parent, wxStri
 }
 
 wxRegEx regexRepeatedChar(wxT(".+[ \\t](<repeats[ \\t][0-9]+[ \\t]times>)$"));
+wxRegEx regexFortranArray(wxT("^\\([0-9,]+)$"));
 
 inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &value, int &start, int length)
 {
     watch->SetDebugValue(value);
     watch->MarkChildsAsRemoved();
 
+    wxString watchSymbol;
+    watch->GetSymbol(watchSymbol);
+    bool isFortranArray = false;
+    if (g_DebugLanguage == dl_Fortran)
+        isFortranArray = regexFortranArray.Matches(watchSymbol);
+    wxString watchSymbolNew;
+    bool hasNewWatchSymbol = false;
+
     int position = start;
     Token token, token_name, token_value;
     wxString pythonToStringValue;
@@ -377,7 +386,7 @@ inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &v
             }
         }
 
-        if (!token.hasRepeatedChar && regexRepeatedChar.Matches(str))
+        if (!token.hasRepeatedChar && regexRepeatedChar.Matches(str) && g_DebugLanguage != dl_Fortran)
         {
             Token expanded_token = token;
             while (1)
@@ -518,8 +527,27 @@ inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &v
                     }
                     else
                     {
-                        int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
-                        cb::shared_ptr<GDBWatch> child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+                        cb::shared_ptr<GDBWatch> child;
+                        if (g_DebugLanguage == dl_Cpp)
+                        {
+                            int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
+                            child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+                        }
+                        else // g_DebugLanguage == dl_Fortran
+                        {
+                            int start_arr = watch->IsArray() ? watch->GetArrayStart() : 1;
+                            wxString childSymbol;
+                            if (isFortranArray)
+                            {
+                                childSymbol = wxString::Format(wxT("(%d,"), start_arr + added_children);
+                                childSymbol << watchSymbol.Mid(1);
+                            }
+                            else
+                            {
+                                childSymbol = wxString::Format(wxT("(%d)"), start_arr + added_children);
+                            }
+                            child = AddChild(watch, childSymbol);
+                        }
                         child->SetValue(token_name.ExtractString(value));
                     }
                     token_name.type = token_value.type = Token::Undefined;
@@ -534,18 +562,65 @@ inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &v
                 cb::shared_ptr<GDBWatch> child;
                 if(token_name.type == Token::Undefined)
                 {
-                    int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
-                    child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+                    if (g_DebugLanguage == dl_Cpp)
+                    {
+                        int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
+                        child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+                    }
+                    else // g_DebugLanguage == dl_Fortran
+                    {
+                        int start_arr = watch->IsArray() ? watch->GetArrayStart() : 1;
+                        wxString childSymbol;
+                        if (isFortranArray)
+                        {
+                            childSymbol = wxString::Format(wxT("(%d,"), start_arr + added_children);
+                            childSymbol << watchSymbol.Mid(1);
+                        }
+                        else
+                        {
+                            childSymbol = wxString::Format(wxT("(%d)"), start_arr + added_children);
+                        }
+                        child = AddChild(watch, childSymbol);
+                    }
                 }
+                else if (g_DebugLanguage == dl_Fortran)
+                    child = AddChild(watch, token_name.ExtractString(value));
                 else
                     child = AddChild(watch, value, token_name);
-                if (!pythonToStringValue.empty())
+
+                if (g_DebugLanguage == dl_Fortran)
+                    child->SetValue(wxEmptyString);
+                else if (!pythonToStringValue.empty())
                     child->SetValue(pythonToStringValue);
                 position = token_real_end;
                 added_children++;
 
                 if(!ParseGDBWatchValue(child, value, position, 0))
                     return false;
+
+                if(g_DebugLanguage == dl_Fortran && isFortranArray && !hasNewWatchSymbol && token_name.type == Token::Undefined)
+                {
+                    // Change watch symbol on the way back for Fortran multidimensional arrays
+                    wxString childSymbol;
+                    child->GetSymbol(childSymbol);
+                    wxString::size_type pos = childSymbol.find_last_of(wxT(':'));
+                    if (pos != wxString::npos)
+                    {
+                        wxString::size_type pos_com = childSymbol.find(wxT(','), pos+2);
+                        if (pos_com != wxString::npos)
+                        {
+                            watchSymbolNew = childSymbol;
+                            watchSymbolNew.replace(pos+2, pos_com-(pos+2), wxT(':'));
+                            hasNewWatchSymbol = true;
+                        }
+                    }
+                    else
+                    {
+                        watchSymbolNew << wxT("(:,") << watchSymbol.Mid(1);
+                        hasNewWatchSymbol = true;
+                    }
+                }
+
                 token_real_end = position;
                 token_name.type = token_value.type = Token::Undefined;
                 skip_comma = true;
@@ -564,9 +639,32 @@ inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &v
                     }
                     else
                     {
-                        int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
-                        cb::shared_ptr<GDBWatch> child = AddChild(watch, wxString::Format(wxT("[%d]"),
-                                                                                          start_arr + added_children));
+                        cb::shared_ptr<GDBWatch> child;
+                        if (g_DebugLanguage == dl_Cpp)
+                        {
+                            int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
+                            child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+                        }
+                        else // g_DebugLanguage == dl_Fortran
+                        {
+                            int start_arr = watch->IsArray() ? watch->GetArrayStart() : 1;
+                            wxString childSymbol;
+                            if (isFortranArray)
+                            {
+                                childSymbol = wxString::Format(wxT("(%d,"), start_arr + added_children);
+                                childSymbol << watchSymbol.Mid(1);
+                                if (!hasNewWatchSymbol)
+                                {
+                                    watchSymbolNew << wxT("(:,") << watchSymbol.Mid(1);
+                                    hasNewWatchSymbol = true;
+                                }
+                            }
+                            else
+                            {
+                                childSymbol = wxString::Format(wxT("(%d)"), start_arr + added_children);
+                            }
+                            child = AddChild(watch, childSymbol);
+                        }
                         child->SetValue(token_name.ExtractString(value));
                     }
                     token_name.type = token_value.type = Token::Undefined;
@@ -576,6 +674,9 @@ inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &v
                     watch->SetValue(wxT(""));
             }
 
+            if (hasNewWatchSymbol)
+                watch->SetSymbol(watchSymbolNew);
+
             start = token_real_end;
             return true;
         case Token::Undefined:
@@ -598,12 +699,23 @@ inline bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &v
         }
         else
         {
-            int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
-            cb::shared_ptr<GDBWatch> child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+            cb::shared_ptr<GDBWatch> child;
+            if (g_DebugLanguage == dl_Cpp)
+            {
+                int start_arr = watch->IsArray() ? watch->GetArrayStart() : 0;
+                child = AddChild(watch, wxString::Format(wxT("[%d]"), start_arr + added_children));
+            }
+            else // g_DebugLanguage == dl_Fortran
+            {
+                int start_arr = watch->IsArray() ? watch->GetArrayStart() : 1;
+                child = AddChild(watch, wxString::Format(wxT("(%d)"), start_arr + added_children));
+            }
             child->SetValue(token_name.ExtractString(value));
         }
     }
 
+
+
     return true;
 }
 
@@ -647,6 +759,14 @@ inline void RemoveBefore(wxString &str, const wxString &s)
     }
 }
 
+void PrepareFortranOutput(wxString& outStr)
+{
+    static wxRegEx nan_line(wxT("nan\\([a-zA-Z0-9]*\\)"));
+    nan_line.Replace(&outStr, wxT("nan"));
+    outStr.Replace(wxT("("),wxT("{"));
+    outStr.Replace(wxT(")"),wxT("}"));
+}
+
 bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &inputValue)
 {
     if(inputValue.empty())
@@ -657,6 +777,9 @@ bool ParseGDBWatchValue(cb::shared_ptr<GDBWatch> watch, wxString const &inputVal
 
     wxString value = RemoveWarnings(inputValue);
 
+    if (g_DebugLanguage == dl_Fortran)
+        PrepareFortranOutput(value);
+
     // Try to find the first brace.
     // If the watch is for a reference the brace is not at position = 0
     wxString::size_type start = value.find(wxT('{'));
@@ -899,3 +1022,57 @@ void TokenizeGDBLocals(std::vector<GDBLocalVariable> &results, wxString const &v
     }
     results.push_back(GDBLocalVariable(value, start, value.length() - start));
 }
+
+const wxRegEx reExamineMemoryLine(wxT("[ \t]*(0x[0-9a-f]+)[ \t]<.+>:[ \t]+(.+)"));
+
+bool ParseGDBExamineMemoryLine(wxString &resultAddr, std::vector<uint8_t> &resultValues,
+                               const wxString &outputLine)
+{
+    // output is a series of:
+    //
+    // 0x22ffc0:       0xf0    0xff    0x22    0x00    0x4f    0x6d    0x81    0x7c
+    // or
+    // 0x85267a0 <RS485TxTask::taskProc()::rcptBuf>:   0x00   0x00   0x00   0x00   0x00   0x00   0x00   0x00
+    // or
+    // > x /108xb 0xa0000000
+    // Cannot access memory at address 0xa0000000
+    // 0xa0000000:
+
+    /// TODO: Detect errors like cannot access memory at address XXX!!!!
+    resultValues.clear();
+    resultAddr.clear();
+
+    if (outputLine.StartsWith(wxT("Cannot access memory at address ")))
+        return false;
+
+    wxString memory;
+    if (reExamineMemoryLine.Matches(outputLine))
+    {
+        resultAddr = reExamineMemoryLine.GetMatch(outputLine, 1);
+        memory = reExamineMemoryLine.GetMatch(outputLine, 2);
+    }
+    else
+    {
+        if (outputLine.First(wxT(':')) == -1)
+        {
+            return false;
+        }
+        resultAddr = outputLine.BeforeFirst(wxT(':'));
+        memory = outputLine.AfterFirst(wxT(':'));
+    }
+
+    size_t pos = memory.find(wxT('x'));
+    wxString hexbyte;
+    while (pos != wxString::npos)
+    {
+        hexbyte.clear();
+        hexbyte << memory[pos + 1];
+        hexbyte << memory[pos + 2];
+        unsigned long value;
+        hexbyte.ToULong(&value,16);
+        resultValues.push_back(static_cast<uint8_t>(value));
+        pos = memory.find(wxT('x'), pos + 1); // skip current 'x'
+    }
+
+    return true;
+}
