@@ -2,9 +2,9 @@
  * This file is part of the Code::Blocks IDE and licensed under the GNU Lesser General Public License, version 3
  * http://www.gnu.org/licenses/lgpl-3.0.html
  *
- * $Revision: 11114 $
- * $Id: cbproject.cpp 11114 2017-07-16 12:23:30Z fuscated $
- * $HeadURL: http://svn.code.sf.net/p/codeblocks/code/branches/release-17.xx/src/sdk/cbproject.cpp $
+ * $Revision: 11905 $
+ * $Id: cbproject.cpp 11905 2019-11-09 12:05:32Z fuscated $
+ * $HeadURL: svn://svn.code.sf.net/p/codeblocks/code/branches/release-20.xx/src/sdk/cbproject.cpp $
  */
 
 #include "sdk_precomp.h"
@@ -117,6 +117,109 @@ cbProject::~cbProject()
     ClearAllProperties();
 }
 
+cbProject& cbProject::operator=(const cbProject &p)
+{
+    CompileTargetBase::operator=(p);
+
+    m_VirtualTargets = p.m_VirtualTargets;
+    m_CurrentlyCompilingTarget = nullptr;
+
+    for (FilesList::iterator it = m_Files.begin(); it != m_Files.end();++it)
+        delete(*it);
+    m_Files.clear();
+
+    typedef std::map<const ProjectFile*, ProjectFile*> OldToNewFilesMap;
+    OldToNewFilesMap oldToNewFiles;
+    for (ProjectFile *oldFile : p.m_Files)
+    {
+        ProjectFile *pf = new ProjectFile(this, *oldFile);
+        m_Files.insert(pf);
+        oldToNewFiles[oldFile] = pf;
+    }
+
+    // Setup the autogenerated files. Probably this step could be moved in the previous loop.
+    for (ProjectFile *src : p.m_Files)
+    {
+        ProjectFile *dst = oldToNewFiles[src];
+        if (src->autoGeneratedBy)
+        {
+            OldToNewFilesMap::const_iterator itGen = oldToNewFiles.find(src->autoGeneratedBy);
+            dst->autoGeneratedBy = ((itGen != oldToNewFiles.end()) ? itGen->second : nullptr);
+        }
+
+        for (ProjectFile *generatedFile : src->generatedFiles)
+        {
+            OldToNewFilesMap::const_iterator itGen = oldToNewFiles.find(generatedFile);
+            if (itGen != oldToNewFiles.end())
+                dst->generatedFiles.push_back(itGen->second);
+        }
+    }
+
+    for (size_t ii = 0; ii < m_Targets.GetCount(); ++ii)
+        delete m_Targets[ii];
+    m_Targets.Clear();
+    for (size_t ii = 0; ii < p.m_Targets.GetCount(); ++ii)
+    {
+        const ProjectBuildTarget *oldTarget = p.m_Targets[ii];
+        ProjectBuildTarget* target = new ProjectBuildTarget(*oldTarget, this);
+        m_Targets.Add(target);
+
+        if (oldTarget == p.m_CurrentlyCompilingTarget)
+            m_CurrentlyCompilingTarget = target;
+
+        // Copy the files for the target.
+        const FilesList &oldFiles = oldTarget->GetFilesList();
+        FilesList &newFiles = target->GetFilesList();
+        for (ProjectFile *pf : oldFiles)
+        {
+            OldToNewFilesMap::const_iterator itNew = oldToNewFiles.find(pf);
+            if (itNew != oldToNewFiles.end())
+                newFiles.insert(itNew->second);
+        }
+    }
+
+    m_ActiveTarget = p.m_ActiveTarget;
+    m_LastSavedActiveTarget = p.m_LastSavedActiveTarget;
+    m_DefaultExecuteTarget = p.m_DefaultExecuteTarget;
+    m_Makefile = p.m_Makefile;
+    m_CustomMakefile = p.m_CustomMakefile;
+    m_MakefileExecutionDir = p.m_MakefileExecutionDir;
+
+    m_Globs = p.m_Globs;
+    m_FileArray.Clear();
+    m_ExpandedNodes = p.m_ExpandedNodes;
+    m_SelectedNodes = p.m_SelectedNodes;
+    m_Loaded = p.m_Loaded;
+    m_ProjectNode = p.m_ProjectNode;
+
+    m_VirtualFolders = p.m_VirtualFolders;
+
+    m_CurrentlyLoading = p.m_CurrentlyLoading;
+    m_CommonTopLevelPath = p.m_CommonTopLevelPath;
+    m_BasePath = p.m_BasePath;
+
+    m_PCHMode = p.m_PCHMode;
+
+    // Just regenerate the hash map for faster lookup of files.
+    m_ProjectFilesMap.clear();
+    for (ProjectFile *pf : m_Files)
+        m_ProjectFilesMap[UnixFilename(pf->relativeFilename)] = pf;
+
+    m_LastModified = p.m_LastModified;
+
+    m_ExtendedObjectNamesGeneration = p.m_ExtendedObjectNamesGeneration;
+    m_Notes = p.m_Notes;
+    m_AutoShowNotesOnLoad = p.m_AutoShowNotesOnLoad;
+    m_CheckForExternallyModifiedFiles = p.m_CheckForExternallyModifiedFiles;
+
+    Delete(m_pExtensionsElement);
+    if (p.m_pExtensionsElement)
+        m_pExtensionsElement = new TiXmlElement(*p.m_pExtensionsElement);
+    else
+        m_pExtensionsElement = nullptr;
+    return *this;
+}
+
 void cbProject::NotifyPlugins(wxEventType type, const wxString& targetName, const wxString& oldTargetName)
 {
     CodeBlocksEvent event(type);
@@ -235,6 +338,10 @@ wxString cbProject::CreateUniqueFilename()
 
 void cbProject::ClearAllProperties()
 {
+    // The macro manager stores pointers to projects and targets, so we need to clear it to prevent
+    // dangling pointer bugs.
+    Manager::Get()->GetMacrosManager()->Reset();
+
     Delete(m_pExtensionsElement);
 
     for (FilesList::iterator it = m_Files.begin(); it != m_Files.end();++it)
@@ -259,6 +366,8 @@ void cbProject::ClearAllProperties()
 
 void cbProject::Open()
 {
+    wxStopWatch timer;
+
     m_Loaded = false;
     m_ProjectFilesMap.clear();
     Delete(m_pExtensionsElement);
@@ -310,6 +419,11 @@ void cbProject::Open()
             m_LastModified = fname.GetModificationTime();
         }
     }
+
+    long time = timer.Time();
+    if (time >= 100)
+        Manager::Get()->GetLogManager()->Log(F(wxT("cbProject::Open took: %.3f seconds."),
+                                               time / 1000.0f));
 } // end of Open
 
 void cbProject::CalculateCommonTopLevelPath()
@@ -424,6 +538,7 @@ wxString cbProject::GetCommonTopLevelPath() const
 void cbProject::Touch()
 {
     m_LastModified = wxDateTime::Now();
+    SetModified(true);
 }
 
 bool cbProject::SaveAs()
@@ -635,7 +750,7 @@ ProjectFile* cbProject::AddFile(int targetIndex, const wxString& filename, bool
 /* NOTE (mandrav#1#): Calling GetFileByFilename() twice, is costly.
     Instead of searching for duplicate files when entering here,
     we 'll search before exiting.
-    The rationale is that by then, we 'll have the relative filename
+    The rationale is that by then, we'll have the relative filename
     in our own representation and this will make everything quicker
     (check GetFileByFilename implementation to understand why)...
 */
@@ -694,7 +809,7 @@ ProjectFile* cbProject::AddFile(int targetIndex, const wxString& filename, bool
         Compiler* c = CompilerFactory::GetCompiler( GetCompilerID() );
         if (c)
         {
-            const CompilerTool* t = c->GetCompilerTool(isResource ? ctCompileResourceCmd : ctCompileObjectCmd, fname.GetExt());
+            const CompilerTool* t = c->GetCompilerTool(isResource ? ctCompileResourceCmd : ctCompileObjectCmd, ext);
             if (t && t->generatedFiles.GetCount())
                 GenFilesHackMap[c] = t;
         }
@@ -707,7 +822,7 @@ ProjectFile* cbProject::AddFile(int targetIndex, const wxString& filename, bool
 
             if (c)
             {
-                const CompilerTool* t = c->GetCompilerTool(isResource ? ctCompileResourceCmd : ctCompileObjectCmd, fname.GetExt());
+                const CompilerTool* t = c->GetCompilerTool(isResource ? ctCompileResourceCmd : ctCompileObjectCmd, ext);
                 if (t && t->generatedFiles.GetCount())
                     GenFilesHackMap[c] = t;
             }
@@ -719,7 +834,7 @@ ProjectFile* cbProject::AddFile(int targetIndex, const wxString& filename, bool
         Compiler* c = CompilerFactory::GetCompiler(m_Targets[targetIndex]->GetCompilerID());
         if (c)
         {
-            const CompilerTool* t = c->GetCompilerTool(isResource ? ctCompileResourceCmd : ctCompileObjectCmd, fname.GetExt());
+            const CompilerTool* t = c->GetCompilerTool(isResource ? ctCompileResourceCmd : ctCompileObjectCmd, ext);
             if (t && t->generatedFiles.GetCount())
                 GenFilesHackMap[c] = t;
         }
@@ -778,13 +893,18 @@ ProjectFile* cbProject::AddFile(int targetIndex, const wxString& filename, bool
     fname.Normalize(wxPATH_NORM_DOTS | wxPATH_NORM_TILDE, projectBasePath);
 
     const wxString &fullFilename = realpath(fname.GetFullPath());
-    pf->file              = fullFilename;
-    pf->relativeFilename  = UnixFilename(local_filename);
+    pf->file = fullFilename;
+
+    // Make sure the relativeFilename is really relative to the project file.
+    // This is a bit slower but a bit more correct.
+    fname.MakeRelativeTo(projectBasePath);
+    const wxString &fixedRelativePath = fname.GetFullPath();
+    pf->relativeFilename = UnixFilename(fixedRelativePath);
 
     // now check if we have already added this file
     // if we have, return the existing file, but add the specified target
     ProjectFile* existing = GetFileByFilename(pf->relativeFilename, true, true);
-    if (existing == pf)
+    if (existing)
     {
         delete pf;
         if (targetIndex >= 0 && targetIndex < (int)m_Targets.GetCount())
@@ -981,7 +1101,7 @@ void cbProject::RestoreTreeState(wxTreeCtrl* tree)
     ::RestoreTreeState(tree, m_ProjectNode, m_ExpandedNodes, m_SelectedNodes);
 }
 
-const wxString& cbProject::GetMakefile()
+const wxString& cbProject::GetMakefile() const
 {
     if (!m_Makefile.IsEmpty())
         return m_Makefile;
@@ -1282,13 +1402,13 @@ bool cbProject::RemoveBuildTarget(int index)
     ProjectBuildTarget* target = GetBuildTarget(index);
     if (target)
     {
-        wxString oldTargetName = target->GetTitle();
+        const wxString targetTitle = target->GetTitle();
 
         // remove target from any virtual targets it belongs to
         for (VirtualBuildTargetsMap::iterator it = m_VirtualTargets.begin(); it != m_VirtualTargets.end(); ++it)
         {
             wxArrayString& tgts = it->second;
-            int virt_idx = tgts.Index(target->GetTitle());
+            int virt_idx = tgts.Index(targetTitle);
             if (virt_idx != -1)
                 tgts.RemoveAt(virt_idx);
         }
@@ -1297,11 +1417,11 @@ bool cbProject::RemoveBuildTarget(int index)
         for (FilesList::iterator it = m_Files.begin(); it != m_Files.end(); ++it)
         {
             ProjectFile* pf = *it;
-            pf->RemoveBuildTarget(target->GetTitle());
+            pf->RemoveBuildTarget(targetTitle);
         }
 
         // notify plugins, before the target is deleted, to make a cleanup possible before the target is really deleted
-        NotifyPlugins(cbEVT_BUILDTARGET_REMOVED, oldTargetName);
+        NotifyPlugins(cbEVT_BUILDTARGET_REMOVED, targetTitle);
         // finally remove the target
         delete target;
         m_Targets.RemoveAt(index);
@@ -1392,12 +1512,25 @@ ProjectBuildTarget* cbProject::GetBuildTarget(int index)
     return nullptr;
 }
 
+const ProjectBuildTarget* cbProject::GetBuildTarget(int index) const
+{
+    if (index >= 0 && index < (int)m_Targets.GetCount())
+        return m_Targets[index];
+    return nullptr;
+}
+
 ProjectBuildTarget* cbProject::GetBuildTarget(const wxString& targetName)
 {
     int idx = IndexOfBuildTargetName(targetName);
     return GetBuildTarget(idx);
 }
 
+const ProjectBuildTarget* cbProject::GetBuildTarget(const wxString& targetName) const
+{
+    int idx = IndexOfBuildTargetName(targetName);
+    return GetBuildTarget(idx);
+}
+
 void cbProject::ReOrderTargets(const wxArrayString& nameOrder)
 {
     LogManager* msgMan = Manager::Get()->GetLogManager();
@@ -1659,6 +1792,11 @@ TiXmlNode* cbProject::GetExtensionsNode()
     return m_pExtensionsElement;
 }
 
+const TiXmlNode* cbProject::GetExtensionsNode() const
+{
+    return m_pExtensionsElement;
+}
+
 void cbProject::AddToExtensions(const wxString& stringDesc)
 {
     // sample stringDesc:
