@@ -2,9 +2,9 @@
  * This file is part of the Code::Blocks IDE and licensed under the GNU Lesser General Public License, version 3
  * http://www.gnu.org/licenses/lgpl-3.0.html
  *
- * $Revision: 11191 $
- * $Id: configmanager.cpp 11191 2017-10-07 13:10:12Z fuscated $
- * $HeadURL: http://svn.code.sf.net/p/codeblocks/code/branches/release-17.xx/src/sdk/configmanager.cpp $
+ * $Revision: 11901 $
+ * $Id: configmanager.cpp 11901 2019-11-04 19:35:26Z fuscated $
+ * $HeadURL: svn://svn.code.sf.net/p/codeblocks/code/branches/release-20.xx/src/sdk/configmanager.cpp $
  */
 
 #include "sdk_precomp.h"
@@ -281,29 +281,28 @@ void CfgMgrBldr::SwitchTo(const wxString& fileName)
     doc->ClearError();
 
     wxString info;
-#ifndef __GNUC__
-    info.Printf(_T( " application info:\n"
+    info.Printf(_T(" application info:\n"
                     "\t svn_revision:\t%u\n"
-                    "\t build_date:\t%s, %s "), ConfigManager::GetRevisionNumber(), wxT(__DATE__), wxT(__TIME__));
-#else
-    info.Printf(_T( " application info:\n"
-                    "\t svn_revision:\t%u\n"
-                    "\t build_date:\t%s, %s\n"
-                    "\t gcc_version:\t%d.%d.%d "), ConfigManager::GetRevisionNumber(), wxT(__DATE__), wxT(__TIME__),
-                __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__);
+                    "\t build_date:\t%s, %s\n"), ConfigManager::GetRevisionNumber(), wxT(__DATE__), wxT(__TIME__));
+#if defined(__clang__)
+    info += wxString::Format(wxT("\t compiler_version:\tclang %d.%d.%d\n"), __clang_major__,
+                             __clang_minor__, __clang_patchlevel__);
+#elif defined(__GNUC__)
+    info += wxString::Format(wxT("\t compiler_version:\tgcc %d.%d.%d\n"), __GNUC__, __GNUC_MINOR__,
+                             __GNUC_PATCHLEVEL__);
 #endif
 
     if (platform::windows)
-        info.append(_T("\n\t Windows "));
-    if (platform::Linux)
-        info.append(_T("\n\t Linux "));
-    if (platform::macosx)
-        info.append(_T("\n\t Mac OS X "));
-    if (platform::Unix)
-        info.append(_T("\n\t Unix "));
-
-    info.append(platform::unicode ? _T("Unicode ") : _T("ANSI "));
+        info.append(_T("\t Windows "));
+    else if (platform::Linux)
+        info.append(_T("\t Linux "));
+    else if (platform::macosx)
+        info.append(_T("\t Mac OS X "));
+    else if (platform::Unix)
+        info.append(_T("\t Unix "));
 
+    info.append(platform::unicode ? _T("Unicode") : _T("ANSI"));
+    info.append(wxT("\n"));
     TiXmlComment c;
     c.SetValue((const char*) info.mb_str());
 
@@ -484,6 +483,11 @@ ConfigManager* CfgMgrBldr::Build(const wxString& name_space)
     return c;
 }
 
+wxString CfgMgrBldr::GetConfigFile() const
+{
+    return cfg;
+}
+
 /*
 *  Hack to enable Turkish language. wxString::Upper will convert lowercase 'i' to \u0130 instead of \u0069 in Turkish locale,
 *  which will break the config file when used in a tag
