@@ -2,9 +2,9 @@
  * This file is part of the Code::Blocks IDE and licensed under the GNU Lesser General Public License, version 3
  * http://www.gnu.org/licenses/lgpl-3.0.html
  *
- * $Revision: 10769 $
- * $Id: editorcolourset.cpp 10769 2016-02-06 14:26:58Z mortenmacfly $
- * $HeadURL: http://svn.code.sf.net/p/codeblocks/code/branches/release-17.xx/src/sdk/editorcolourset.cpp $
+ * $Revision: 11482 $
+ * $Id: editorcolourset.cpp 11482 2018-09-29 12:20:40Z fuscated $
+ * $HeadURL: svn://svn.code.sf.net/p/codeblocks/code/branches/release-20.xx/src/sdk/editorcolourset.cpp $
  */
 
 #include "sdk_precomp.h"
@@ -32,9 +32,6 @@
 #include "editorlexerloader.h"
 #include "filefilters.h"
 
-const int cbHIGHLIGHT_LINE = -98; // highlight line under caret virtual style
-const int cbSELECTION      = -99; // selection virtual style
-
 EditorColourSet::EditorColourSet(const wxString& setName)
     : m_Name(setName)
 {
@@ -46,9 +43,10 @@ EditorColourSet::EditorColourSet(const wxString& setName)
         Load();
 }
 
-EditorColourSet::EditorColourSet(const EditorColourSet& other) // copy ctor
+EditorColourSet::EditorColourSet(const EditorColourSet& other) :// copy ctor
+    m_Name(other.m_Name),
+    m_PlainTextLexerID(other.m_PlainTextLexerID)
 {
-    m_Name = other.m_Name;
     m_Sets.clear();
 
     for (OptionSetsMap::const_iterator it = other.m_Sets.begin(); it != other.m_Sets.end(); ++it)
@@ -181,7 +179,7 @@ void EditorColourSet::LoadAvailableSets()
 
 HighlightLanguage EditorColourSet::AddHighlightLanguage(int lexer, const wxString& name)
 {
-    if (   lexer <= wxSCI_LEX_NULL
+    if (   lexer < wxSCI_LEX_NULL
         || lexer >  wxSCI_LEX_LAST // this is a C::B extension to wxscintilla.h
         || name.IsEmpty() )
     {
@@ -210,6 +208,10 @@ HighlightLanguage EditorColourSet::AddHighlightLanguage(int lexer, const wxStrin
 
     m_Sets[newID].m_Langs = name;
     m_Sets[newID].m_Lexers = lexer;
+
+    if (lexer == wxSCI_LEX_NULL && name == wxT("Plain text files"))
+        m_PlainTextLexerID = newID;
+
     return newID;
 }
 
@@ -246,7 +248,9 @@ wxArrayString EditorColourSet::GetAllHighlightLanguages()
     wxArrayString ret;
     for (OptionSetsMap::iterator it = m_Sets.begin(); it != m_Sets.end(); ++it)
     {
-        if (!it->second.m_Langs.IsEmpty())
+        // Make sure to skip the "plain text files", because we don't want it in the list. We
+        // manually add "Plain text" to the appropriate places.
+        if (!it->second.m_Langs.IsEmpty() && it->first != m_PlainTextLexerID)
             ret.Add(it->second.m_Langs);
     }
     ret.Sort(CompareStringNoCase);
@@ -291,7 +295,7 @@ void EditorColourSet::UpdateOptionsWithSameName(HighlightLanguage lang, OptionCo
 bool EditorColourSet::AddOption(HighlightLanguage lang, OptionColour* option, bool checkIfExists)
 {
     if (lang == HL_NONE)
-        return false;
+        lang = m_PlainTextLexerID;
 
     if (checkIfExists && GetOptionByValue(lang, option->value))
         return false;
@@ -312,7 +316,7 @@ void EditorColourSet::AddOption(HighlightLanguage lang,
                                 bool              isStyle)
 {
     if (lang == HL_NONE)
-        return;
+        lang = m_PlainTextLexerID;
 
     OptionColour* opt = new OptionColour;
     opt->name = name;
@@ -338,7 +342,7 @@ void EditorColourSet::AddOption(HighlightLanguage lang,
 OptionColour* EditorColourSet::GetOptionByName(HighlightLanguage lang, const wxString& name)
 {
     if (lang == HL_NONE)
-        return nullptr;
+        lang = m_PlainTextLexerID;
 
     OptionSet& mset = m_Sets[lang];
     for (unsigned int i = 0; i < mset.m_Colours.GetCount(); ++i)
@@ -353,7 +357,7 @@ OptionColour* EditorColourSet::GetOptionByName(HighlightLanguage lang, const wxS
 OptionColour* EditorColourSet::GetOptionByValue(HighlightLanguage lang, int value)
 {
     if (lang == HL_NONE)
-        return nullptr;
+        lang = m_PlainTextLexerID;
 
     OptionSet& mset = m_Sets[lang];
     for (unsigned int i = 0; i < mset.m_Colours.GetCount(); ++i)
@@ -368,13 +372,16 @@ OptionColour* EditorColourSet::GetOptionByValue(HighlightLanguage lang, int valu
 OptionColour* EditorColourSet::GetOptionByIndex(HighlightLanguage lang, int index)
 {
     if (lang == HL_NONE)
-        return nullptr;
+        return m_Sets[m_PlainTextLexerID].m_Colours.Item(index);
     return m_Sets[lang].m_Colours.Item(index);
 }
 
 int EditorColourSet::GetOptionCount(HighlightLanguage lang)
 {
-    return m_Sets[lang].m_Colours.GetCount();
+    if (lang != HL_NONE)
+        return m_Sets[lang].m_Colours.GetCount();
+    else
+        return m_Sets[m_PlainTextLexerID].m_Colours.GetCount();
 }
 
 // Encapsulate the getter for the default option. The default option
@@ -383,7 +390,7 @@ int EditorColourSet::GetOptionCount(HighlightLanguage lang)
 OptionColour* EditorColourSet::GetDefaultOption(HighlightLanguage lang)
 {
     if (lang == HL_NONE)
-        return nullptr;
+        lang = m_PlainTextLexerID;
 
     OptionSet& mset = m_Sets[lang];
     OptionColour *defaultOpt = nullptr;
@@ -464,17 +471,18 @@ HighlightLanguage EditorColourSet::GetLanguageForFilename(const wxString& filena
             "ctime|"     "cuchar|"  "cwchar|"  "cwctype|"            );
     if (cppNames.Find(wxT("|") + lfname + wxT("|")) != wxNOT_FOUND)
         return GetHighlightLanguage(wxT("C/C++"));
+
     return HL_NONE;
 }
 
 wxString EditorColourSet::GetLanguageName(HighlightLanguage lang)
 {
-    if (lang == HL_NONE)
-        return _("Plain text");
-
-    wxString name = m_Sets[lang].m_Langs;
-    if (!name.IsEmpty())
-        return name;
+    if (lang != HL_NONE && lang != m_PlainTextLexerID)
+    {
+        wxString name = m_Sets[lang].m_Langs;
+        if (!name.empty())
+            return name;
+    }
 
     return _("Plain text");
 }
@@ -492,7 +500,7 @@ void EditorColourSet::DoApplyStyle(cbStyledTextCtrl* control, int value, OptionC
     control->StyleSetUnderline(value, option->underlined);
 }
 
-HighlightLanguage EditorColourSet::Apply(cbEditor* editor, HighlightLanguage lang)
+HighlightLanguage EditorColourSet::Apply(cbEditor* editor, HighlightLanguage lang, bool colourise)
 {
     if (!editor)
         return HL_NONE;
@@ -504,25 +512,48 @@ HighlightLanguage EditorColourSet::Apply(cbEditor* editor, HighlightLanguage lan
                       && lang == GetHighlightLanguage(wxT("C/C++"))
                       && editor->GetFilename().Lower().EndsWith(wxT(".c")) );
 
-    Apply(lang, editor->GetLeftSplitViewControl(),  isC);
-    Apply(lang, editor->GetRightSplitViewControl(), isC);
+    Apply(lang, editor->GetLeftSplitViewControl(),  isC, colourise);
+    Apply(lang, editor->GetRightSplitViewControl(), isC, colourise);
 
     return lang;
 }
 
 
-void EditorColourSet::Apply(HighlightLanguage lang, cbStyledTextCtrl* control, bool isC)
+
+// Encapsulate the getter for the default option. The default option
+// is the option named "Default" and if there is no such option we use
+// the option with value/index equal to 0.
+static OptionColour* GetDefaultOption(OptionSet& mset)
+{
+    OptionColour *defaultOpt = nullptr;
+    for (size_t i = 0; i < mset.m_Colours.GetCount(); ++i)
+    {
+        OptionColour* opt = mset.m_Colours.Item(i);
+        if (opt->name == wxT("Default"))
+            return opt;
+        if (opt->value == 0)
+            defaultOpt = opt;
+    }
+    return defaultOpt;
+}
+
+void EditorColourSet::Apply(HighlightLanguage lang, cbStyledTextCtrl* control, bool isC,
+                            bool colourise)
 {
     if (!control)
         return;
     control->StyleClearAll();
 
     if (lang == HL_NONE)
-        return;
+    {
+        if (m_PlainTextLexerID.empty())
+            return;
+        lang = m_PlainTextLexerID;
+    }
 
     // first load the default colours to all styles used by the actual lexer (ignoring some built-in styles)
-    OptionColour* defaults = GetDefaultOption(lang);
     OptionSet& mset = m_Sets[lang];
+    OptionColour* defaults = ::GetDefaultOption(mset);
     control->SetLexer(mset.m_Lexers);
     control->SetStyleBits(control->GetStyleBitsNeeded());
     if (defaults)
@@ -555,7 +586,6 @@ void EditorColourSet::Apply(HighlightLanguage lang, cbStyledTextCtrl* control, b
             if (opt->value == cbHIGHLIGHT_LINE)
             {
                 control->SetCaretLineBackground(opt->back);
-                Manager::Get()->GetConfigManager(_T("editor"))->Write(_T("/highlight_caret_line_colour"), opt->back);
             }
             else if (opt->value == cbSELECTION)
             {
@@ -565,7 +595,7 @@ void EditorColourSet::Apply(HighlightLanguage lang, cbStyledTextCtrl* control, b
 //                    Manager::Get()->GetConfigManager(_T("editor"))->Write(_T("/selection_colour"), opt->back);
                 }
                 else
-                    control->SetSelBackground(false, wxColour(0xC0, 0xC0, 0xC0));
+                    control->SetSelBackground(false, wxColour(0xD9, 0xD9, 0xD9));
 
                 if (opt->fore != wxNullColour)
                 {
@@ -588,7 +618,8 @@ void EditorColourSet::Apply(HighlightLanguage lang, cbStyledTextCtrl* control, b
             control->SetKeyWords(i, mset.m_Keywords[i]);
     }
 
-    control->Colourise(0, -1); // the *most* important part!
+    if (colourise)
+        control->Colourise(0, -1); // the *most* important part!
 }
 
 void EditorColourSet::Save()
@@ -850,19 +881,19 @@ const wxArrayString& EditorColourSet::GetFileMasks(HighlightLanguage lang)
 
 void EditorColourSet::SetFileMasks(HighlightLanguage lang, const wxString& masks, const wxString& separator)
 {
-    if (lang != HL_NONE)
-    {
-        m_Sets[lang].m_FileMasks = GetArrayFromString(masks.Lower(), separator);
+    if (lang == HL_NONE)
+        lang = m_PlainTextLexerID;
 
-        // let's add these filemasks in the file filters master list ;)
-        FileFilters::Add(wxString::Format(_("%s files"), m_Sets[lang].m_Langs.c_str()), masks);
-    }
+    m_Sets[lang].m_FileMasks = GetArrayFromString(masks.Lower(), separator);
+
+    // let's add these filemasks in the file filters master list ;)
+    FileFilters::Add(wxString::Format(_("%s files"), m_Sets[lang].m_Langs.c_str()), masks);
 }
 
 wxString EditorColourSet::GetSampleCode(HighlightLanguage lang, int* breakLine, int* debugLine, int* errorLine)
 {
     if (lang == HL_NONE)
-        return wxEmptyString;
+        lang = m_PlainTextLexerID;
     OptionSet& mset = m_Sets[lang];
     if (breakLine)
         *breakLine = mset.m_BreakLine;
@@ -894,7 +925,7 @@ wxString EditorColourSet::GetSampleCode(HighlightLanguage lang, int* breakLine,
 void EditorColourSet::SetSampleCode(HighlightLanguage lang, const wxString& sample, int breakLine, int debugLine, int errorLine)
 {
     if (lang == HL_NONE)
-        return;
+        lang = m_PlainTextLexerID;
     OptionSet& mset = m_Sets[lang];
     mset.m_SampleCode = sample;
     mset.m_BreakLine = breakLine;
@@ -905,7 +936,8 @@ void EditorColourSet::SetSampleCode(HighlightLanguage lang, const wxString& samp
 void EditorColourSet::SetCommentToken(HighlightLanguage lang, CommentToken token)
 {
     if (lang == HL_NONE)
-        return;
+        lang = m_PlainTextLexerID;
+
     m_Sets[lang].comment = token;
 }
 
@@ -931,7 +963,7 @@ CommentToken EditorColourSet::GetCommentToken(HighlightLanguage lang)
 void EditorColourSet::SetCaseSensitivity(HighlightLanguage lang, bool CaseSensitive)
 {
     if ( lang == HL_NONE )
-        return;
+        lang = m_PlainTextLexerID;
 
     m_Sets[lang].m_CaseSensitive = CaseSensitive;
 }
@@ -939,7 +971,7 @@ void EditorColourSet::SetCaseSensitivity(HighlightLanguage lang, bool CaseSensit
 bool EditorColourSet::GetCaseSensitivity(HighlightLanguage lang)
 {
     if ( lang == HL_NONE )
-        return false;
+        lang = m_PlainTextLexerID;
 
     return m_Sets[lang].m_CaseSensitive;
 }
@@ -947,7 +979,7 @@ bool EditorColourSet::GetCaseSensitivity(HighlightLanguage lang)
 void EditorColourSet::SetStringLexerStyles(HighlightLanguage lang, const std::set<int> &styles)
 {
     if ( lang == HL_NONE )
-        return;
+        lang = m_PlainTextLexerID;
 
    cbStyledTextCtrl::GetStringLexerStyles()[m_Sets[lang].m_Lexers] = styles;
 }
@@ -955,15 +987,15 @@ void EditorColourSet::SetStringLexerStyles(HighlightLanguage lang, const std::se
 void EditorColourSet::SetCommentLexerStyles(HighlightLanguage lang, const std::set<int> &styles)
 {
     if ( lang == HL_NONE )
-        return;
+        lang = m_PlainTextLexerID;
 
-   cbStyledTextCtrl::GetCommentLexerStyles()[m_Sets[lang].m_Lexers] = styles;
+    cbStyledTextCtrl::GetCommentLexerStyles()[m_Sets[lang].m_Lexers] = styles;
 }
 
 void EditorColourSet::SetCharacterLexerStyles(HighlightLanguage lang, const std::set<int> &styles)
 {
     if ( lang == HL_NONE )
-        return;
+        lang = m_PlainTextLexerID;
 
    cbStyledTextCtrl::GetCharacterLexerStyles()[m_Sets[lang].m_Lexers] = styles;
 }
@@ -971,7 +1003,7 @@ void EditorColourSet::SetCharacterLexerStyles(HighlightLanguage lang, const std:
 void EditorColourSet::SetPreprocessorLexerStyles(HighlightLanguage lang, const std::set<int> &styles)
 {
     if ( lang == HL_NONE )
-        return;
+        lang = m_PlainTextLexerID;
 
    cbStyledTextCtrl::GetPreprocessorLexerStyles()[m_Sets[lang].m_Lexers] = styles;
 }
